package armazenamento;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.util.Vector;

import util.IteratorBidirecional;
import dados.Entrega;

public class RepositorioEntrega {
	
	private static RepositorioEntrega instancia;
	private File arquivo;
	private Vector<Entrega> entregas;

	static{
		RepositorioEntrega.instancia = new RepositorioEntrega();
	}

	private RepositorioEntrega(){
		this.entregas = new Vector<Entrega>();
	}

	public static RepositorioEntrega getInstancia(){
		if(instancia.arquivo == null){
			instancia.arquivo = new File("entregas.txt");

			try {
				instancia.entregas.clear();
				FileInputStream finput = new FileInputStream(instancia.arquivo);
				ObjectInputStream input = new ObjectInputStream(finput);

				int qtd = input.readInt();

				for(int i = 0; i < qtd; i++){
					instancia.entregas.add((Entrega) input.readObject());
				}

				input.close();
				finput.close();
			} catch (Exception e) {
				PrintWriter p;
				
				try {
					p = new PrintWriter(instancia.arquivo);
					p.close();
				} catch (FileNotFoundException e1) {
				}
			}
		}

		return RepositorioEntrega.instancia;
	}

	public void inserirEntrega(Entrega entrega){
		this.entregas.add(entrega);
		this.atualizarArquivo();
	}
	
	public void removerEntrega(int codigoCopia){
		boolean achou = false;
		
		for(int i = 0; i < this.entregas.size() && !achou; i++){
			if(this.entregas.get(i).getLocacao().getCopia().getCodigoCopia() == codigoCopia){
				this.entregas.remove(i);
				achou = true;
			}
		}
		
		if(achou){
			this.atualizarArquivo();
		}
	}

	private void atualizarArquivo() {
		try {
			FileOutputStream fout = null;

			try{
				fout = new FileOutputStream(this.arquivo);
			} catch(FileNotFoundException e){
				this.arquivo.createNewFile();

				fout = new FileOutputStream(this.arquivo);
			}
			
			ObjectOutputStream out = new ObjectOutputStream(fout);
			out.writeInt(this.entregas.size());
			
			for(int i = 0; i < this.entregas.size(); i++){
				out.writeObject(this.entregas.get(i));
			}
			
			out.close();
			fout.close();
		} catch (FileNotFoundException e) {
		} catch (IOException e) {
		}
		
	}

	public IteratorBidirecional<Entrega> getEntregas() {
		return new IteratorBidirecional<Entrega>(this.entregas.iterator());
	}

}
