package armazenamento;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.util.Iterator;
import java.util.Vector;

import util.IteratorBidirecional;
import dados.Operacao;

public class RepositorioOperacao {

	private static RepositorioOperacao instancia;

	private long codigos;
	private File arquivo;
	private Vector<Operacao> operacoes;

	static{
		RepositorioOperacao.instancia = new RepositorioOperacao();
	}

	private RepositorioOperacao(){
		this.codigos = 0;
		this.operacoes = new Vector<Operacao>();
	}

	public static RepositorioOperacao getInstancia() {
		if(instancia.arquivo == null){
			instancia.arquivo = new File("operacoes/OperacoesPendentes.op");

			try {
				FileInputStream finput = new FileInputStream(instancia.arquivo);
				ObjectInputStream input = new ObjectInputStream(finput);
				instancia.operacoes.clear();
				
				int qtd = input.readInt();

				for(int i = 0; i < qtd; i++){
					instancia.operacoes.add((Operacao) input.readObject());
				}

				input.close();
				finput.close();
			} catch (Exception e) {
				try {
					PrintWriter out = new PrintWriter(instancia.arquivo);
					out.close();
				} catch (FileNotFoundException e1) {
				}
			}
		}

		return RepositorioOperacao.instancia;
	}

	public void inserirOperacao(Operacao op) throws IOException {
		if(op != null){
			op.setCodigo(this.codigos++);
			this.operacoes.add(op);

			try {
				this.atualizarArquivo();
			} catch (IOException e) {
				this.arquivo = null;
				throw e;
			}
		} else {
			throw new IllegalArgumentException();
		}
	}

	public boolean existeOperacao(Operacao op){
		if(op != null){
			Iterator<Operacao> it = this.operacoes.iterator();
			boolean retorno = false;

			while(it.hasNext() && !retorno){
				if(it.next().equals(op)){
					retorno = true;
				}
			}

			return retorno;
		} else {
			throw new IllegalArgumentException();
		}
	}

	public void removerOperacao(Operacao op) throws IOException{
		if(op != null){
			boolean achou = false;

			for(int i = this.operacoes.size()-1; i >= 0 && !achou; i--){
				if(this.operacoes.get(i).equals(op)){
					this.operacoes.remove(i);
					achou = true;
				}
			}

			try {
				this.atualizarArquivo();
			} catch (IOException e) {
				this.arquivo = null;
				throw e;
			}
		} else {
			throw new IllegalArgumentException();
		}
	}

	public IteratorBidirecional<Operacao> getOperacoes(){
		return new IteratorBidirecional<Operacao>(operacoes.iterator());
	}

	private void atualizarArquivo() throws IOException {
		FileOutputStream fout = null;

		try{
			fout = new FileOutputStream(this.arquivo);
		} catch(FileNotFoundException e){
			this.arquivo.createNewFile();

			fout = new FileOutputStream(this.arquivo);
		}

		ObjectOutputStream out = new ObjectOutputStream(fout);

		int qtd = this.operacoes.size();

		out.writeInt(qtd);
		for(int i = 0; i < qtd; i++){
			out.writeObject(this.operacoes.get(i));
		}

		out.close();
		fout.close();
	}

}