package armazenamento;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Vector;

import dados.exception.LinkFotoInvalidoException;
import dados.exception.PlanoInvalidoException;
import dados.produto.LinkFoto;
import dados.produto.TipoLinkFoto;
import dados.usuario.Plano;

import util.IteratorBidirecional;


public class RepositorioPlanoOffline {

	private static RepositorioPlanoOffline instancia;
	private File arquivo;
	private Vector<Plano> planos;

	static{
		RepositorioPlanoOffline.instancia = new RepositorioPlanoOffline();
	}

	private RepositorioPlanoOffline(){
		planos = new Vector<Plano>();
	}

	public static RepositorioPlanoOffline getInstancia(){
		if(instancia.arquivo == null){
			instancia.arquivo = new File("planos.txt");

			try {
				FileInputStream finput = new FileInputStream(instancia.arquivo);
				ObjectInputStream input = new ObjectInputStream(finput);
				instancia.planos.clear();

				int qtd = input.readInt();

				for(int i = 0; i < qtd; i++){
					instancia.planos.add((Plano) input.readObject());
				}

				finput.close();
				input.close();
			} catch (Exception e) {
				if(instancia.planos.isEmpty()){
					try {
						instancia.planos.add(new Plano(0.0, "Plano Bsico", "temporrio", 0, new LinkFoto(TipoLinkFoto.NORMAL, "broken")));
					} catch (PlanoInvalidoException e1) {
					} catch (LinkFotoInvalidoException e1) {
					}
				}
			}
		}

		return RepositorioPlanoOffline.instancia;
	}

	public IteratorBidirecional<Plano> getPlanos(){
		return new IteratorBidirecional<Plano>(this.planos.iterator());
	}
	
	public void atualizarPlanos(IteratorBidirecional<Plano> planos){
		this.planos.clear();
		
		while(planos.hasNext()){
			this.planos.add(planos.next());
		}
		
		this.atualizarArquivo();
	}

	private void atualizarArquivo() {
		try {
			FileOutputStream fout = null;

			try{
				fout = new FileOutputStream(this.arquivo);
			} catch(FileNotFoundException e){
				this.arquivo.createNewFile();

				fout = new FileOutputStream(this.arquivo);
			}
			
			ObjectOutputStream out = new ObjectOutputStream(fout);
			out.writeInt(this.planos.size());
			
			for(int i = 0; i < this.planos.size(); i++){
				out.writeObject(this.planos.get(i));
			}
			
			out.close();
			fout.close();
		} catch (FileNotFoundException e) {
		} catch (IOException e) {
		}
		
	}

}
