package fachada;

import gui.JanelaFilial;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Scanner;
import java.util.Vector;

import negocio.GerenciadorEntregas;
import negocio.GerenciadorOperacoes;
import negocio.GerenciadorPlanoOffline;

import dados.Entrega;
import dados.Operacao;
import dados.TipoOperacao;
import dados.exception.CpfInvalidoException;
import dados.exception.EmailInvalidoException;
import dados.exception.FilialInvalidaException;
import dados.exception.LocacaoInvalidaException;
import dados.exception.NomeInvalidoException;
import dados.exception.OperacaoInvalidaException;
import dados.exception.ReservaInvalidaException;
import dados.locacao.Copia;
import dados.locacao.Locacao;
import dados.locacao.Periodo;
import dados.locacao.Reserva;
import dados.produto.Produto;
import dados.usuario.Cliente;
import dados.usuario.Dependente;
import dados.usuario.Funcionario;
import dados.usuario.Plano;
import dados.usuario.Telefone;
import dados.usuario.Usuario;
import negocio.exception.CopiaInexistenteException;
import negocio.exception.CopiaLocadaNoPeriodoException;
import negocio.exception.CopiaReservadaNoPeriodoException;
import negocio.exception.EntregaInexistenteException;
import negocio.exception.ErroValidacaoException;
import negocio.exception.FilialInexistenteException;
import negocio.exception.FilialLogadaException;
import negocio.exception.FilialNaoLogadaException;
import negocio.exception.LocacaoExistenteException;
import negocio.exception.LocacaoInexistenteException;
import negocio.exception.OperacaoExistenteException;
import negocio.exception.OperacaoInexistenteException;
import negocio.exception.PlanoInexistenteException;
import negocio.exception.ProdutoInexistenteException;
import negocio.exception.ReservaExistenteException;
import negocio.exception.ReservaInexistenteException;
import negocio.exception.SenhaInvalidaException;
import negocio.exception.UsuarioExistenteException;
import negocio.exception.UsuarioInexistenteException;
import negocio.exception.UsuarioLogadoException;
import negocio.exception.UsuarioNaoLogadoException;
import negocio.exception.UsuarioSemPermissaoException;
import servicos.InterfaceFilial;
import servicos.InterfaceFilialBD;
import servicos.InterfaceFilialImp;
import util.IteratorBidirecional;

public class Fachada {

	private JanelaFilial janelaPrincipal;
	private InterfaceFilialBD servico;
	private static Fachada instancia;
	private GerenciadorOperacoes gerenciadorOperacoes;
	private GerenciadorPlanoOffline gerenciadorPlanoOffline;
	private GerenciadorEntregas gerenciadorEntregas;
	private Long sessao;
	private String senhaFuncionario;
	private String filial;

	private ArrayList<Reserva> reservasNotificadas;

	static{
		Fachada.instancia = new Fachada();
	}

	private Fachada() {
		this.reservasNotificadas = new ArrayList<Reserva>();
		Scanner scanner;
		try {
			scanner = new Scanner(new File("filial.txt"));
			this.filial = scanner.nextLine();
			scanner.close();		
		} catch (FileNotFoundException e) {
			this.filial = "";
		}
	}

	public static Fachada getInstancia(JanelaFilial janela){
		if(janela != null){
			instancia.janelaPrincipal = janela;
		}

		if(instancia.gerenciadorOperacoes == null){
			instancia.gerenciadorOperacoes = GerenciadorOperacoes.getInstancia();
		}

		if(instancia.gerenciadorPlanoOffline == null){
			instancia.gerenciadorPlanoOffline = GerenciadorPlanoOffline.getInstancia();
		}

		if(instancia.gerenciadorEntregas == null){
			instancia.gerenciadorEntregas = GerenciadorEntregas.getInstancia();
		}

		return Fachada.instancia;
	}

	public boolean existeOperacao(){
		return this.gerenciadorOperacoes.existeOperacao();
	}

	public IteratorBidirecional<Operacao> getOperacoes() throws OperacaoInexistenteException{
		return this.gerenciadorOperacoes.getOperacoes();
	}

	public void logar(long cpf, String senha) throws RemoteException, UsuarioLogadoException, UsuarioInexistenteException, SenhaInvalidaException, FilialInexistenteException {
		if(this.servico == null){
			try{
				String ipConfig = String.format("rmi://%s:1099/",InterfaceFilialBD.ip);
				this.servico = (InterfaceFilialBD) Naming.lookup(ipConfig +	"InterfaceFilialBD");
			} catch(MalformedURLException e){
				// NAO deve ocorrer
			} catch(NotBoundException e){
				throw new RemoteException();
			}
		}


		try{
			this.sessao = this.servico.logarUsuario(cpf, senha);
		} catch(UsuarioLogadoException e){
			try {
				if(this.sessao != null){
					this.servico.deslogar(this.sessao);
				}

				this.sessao = this.servico.logarUsuario(cpf, senha);
			} catch (UsuarioNaoLogadoException e1) {
				throw new UsuarioLogadoException(null);
			}
		}

		try {
			Usuario user = this.servico.buscarUsuario(this.sessao, cpf);

			if(!(user instanceof Funcionario)){
				this.servico.deslogar(this.sessao);
				this.sessao = null;
				this.servico = null;

				throw new UsuarioSemPermissaoException(user);
			}
		} catch (UsuarioNaoLogadoException e) {
			// NAO deve ocorrer
		} catch (UsuarioSemPermissaoException e) {
			// NAO deve ocorrer
		}

		this.senhaFuncionario = senha;
	}

	public void deslogar() throws UsuarioNaoLogadoException, FilialNaoLogadaException{
		try {
			if(this.servico != null){
				try{
					this.servico.deslogarFilial(this.sessao, this.filial);
				}catch (FilialNaoLogadaException e1){
				} catch (FilialInexistenteException e) {
				} catch (UsuarioSemPermissaoException e) {
				}

				this.servico.deslogar(this.sessao);
				this.sessao = null;
				this.servico = null;
			} else {
				throw new FilialNaoLogadaException(this.filial);
			}
		} catch (RemoteException e) {
			this.servico = null;
			throw new FilialNaoLogadaException(this.filial);
		}
	}

	public void logarFilial() throws UsuarioNaoLogadoException, FilialLogadaException, FilialInexistenteException, IOException, FilialNaoLogadaException, UsuarioSemPermissaoException{
		try{
			if(this.sessao != null && this.servico != null){
				if(this.filial.isEmpty() || this.filial == null){
					Scanner scanner = new Scanner(new File("filial.txt"));
					this.filial = scanner.nextLine();
					scanner.close();
				}

				this.servico.logarFilial(this.sessao, this.filial);

				int port = 1099;
				Registry registry = LocateRegistry.getRegistry(InterfaceFilialBD.ip, port);

				if ( System.getSecurityManager() == null ){
					System.setSecurityManager(new RMISecurityManager());
				}

				String name = "InterfaceFilial_" + this.gerarNome(this.filial);
				InterfaceFilial obj = new InterfaceFilialImp();

				registry.rebind(name, obj);
				System.out.println( " Interface Filial registrada. ");
			} else {
				throw new UsuarioNaoLogadoException();
			}
		} catch (RemoteException e) {
			this.deslogarFilial();
			this.servico = null;
			throw new FilialNaoLogadaException(this.filial);
		}
	}

	private String gerarNome(String nome){
		for(int i = 0; i < nome.length(); i++){
			if(nome.charAt(i) == ' '){
				nome = nome.substring(0, i) + nome.substring(i+1, nome.length());
			}
		}

		return nome;
	}

	public void deslogarFilial() throws FilialNaoLogadaException, FilialInexistenteException, UsuarioNaoLogadoException{
		try{
			if(this.sessao != null && this.servico != null){
				try {
					this.servico.deslogarFilial(this.sessao, this.filial);
				} catch (UsuarioSemPermissaoException e) {
					// NAO deve ocorrer
				}
			} else {
				throw new UsuarioNaoLogadoException();
			}
		} catch (RemoteException e) {
			this.servico = null;
			throw new FilialNaoLogadaException(this.filial);
		}
	}

	public void cadastrarCliente(Cliente c, Telefone t) throws UsuarioSemPermissaoException, UsuarioExistenteException, UsuarioNaoLogadoException, OperacaoExistenteException, OperacaoInvalidaException, IOException{
		Telefone tel[] = new Telefone[1];
		tel[0] = t;

		c.setTelefones(tel);
		this.cadastrarUsuario(c);
	}

	public void cadastrarDependente(long cpf, String senha, String nome, String email, long cpfCliente) throws UsuarioNaoLogadoException, UsuarioInexistenteException, UsuarioSemPermissaoException, UsuarioExistenteException, NomeInvalidoException, SenhaInvalidaException, CpfInvalidoException, EmailInvalidoException, OperacaoExistenteException, IOException, OperacaoInvalidaException{
		try {
			if(this.servico != null){
				Usuario c = this.servico.buscarUsuario(this.sessao, cpfCliente);
				if(c instanceof Cliente){
					this.servico.cadastrarUsuario(this.sessao, new Dependente(cpf, senha, nome, email, (Cliente) c));
				} else {
					throw new UsuarioInexistenteException();
				}
			} else {
				throw new RemoteException();
			}
		} catch (RemoteException e) {
			this.servico = null;

			Vector<Object> parametros = new Vector<Object>();
			parametros.add(cpf);
			parametros.add(senha);
			parametros.add(nome);
			parametros.add(email);
			parametros.add(cpfCliente);
			this.gerenciadorOperacoes.inserirOperacao(new Operacao(TipoOperacao.CADASTRARDEPENDENTE, parametros));
		}
	}

	private void cadastrarUsuario(Usuario user) throws UsuarioSemPermissaoException, UsuarioExistenteException, UsuarioNaoLogadoException, OperacaoExistenteException, OperacaoInvalidaException, IOException{
		try {
			if(this.servico != null){
				this.servico.cadastrarUsuario(this.sessao, user);
			} else {
				throw new RemoteException();
			}
		} catch (RemoteException e) {
			this.servico = null;

			Vector<Object> parametros = new Vector<Object>();
			parametros.add(user);
			this.gerenciadorOperacoes.inserirOperacao(new Operacao(TipoOperacao.CADASTRARCLIENTE, parametros));
		}
	}

	public Usuario buscarUsuario(long cpf) throws UsuarioNaoLogadoException, UsuarioInexistenteException, UsuarioSemPermissaoException, FilialNaoLogadaException{
		try{
			if(this.servico != null){

				return this.servico.buscarUsuario(this.sessao, cpf);
			} else {
				throw new FilialNaoLogadaException(this.filial);
			}
		} catch (RemoteException e) {
			this.servico = null;
			throw new FilialNaoLogadaException(this.filial);	
		}
	}

	public void alterarDadosUsuario(Usuario dados) throws UsuarioNaoLogadoException, UsuarioInexistenteException, UsuarioExistenteException, UsuarioSemPermissaoException, OperacaoExistenteException, IOException, OperacaoInvalidaException, SenhaInvalidaException{
		try{
			if(this.servico != null){
				Usuario user = this.servico.buscarUsuario(this.sessao, dados.getCpf());
				if(dados.getSenha().equals(user.getSenha())){
					this.servico.alterarDadosUsuario(this.sessao, dados);
				} else {
					throw new SenhaInvalidaException();
				}
			} else {
				throw new RemoteException();
			}
		} catch (RemoteException e) {
			this.servico = null;
			Vector<Object> parametros = new Vector<Object>();
			parametros.add(dados);
			this.gerenciadorOperacoes.inserirOperacao(new Operacao(TipoOperacao.ALTERARDADOSUSUARIO, parametros));
		}
	}

	public void descadastrarUsuario(long cpfUsuario, String senha) throws UsuarioInexistenteException, UsuarioSemPermissaoException, UsuarioNaoLogadoException, SenhaInvalidaException, OperacaoExistenteException, IOException, OperacaoInvalidaException{
		try{
			if(this.servico != null){
				Usuario user = this.servico.buscarUsuario(this.sessao, cpfUsuario);
				if(senha.equals(user.getSenha())){
					this.servico.descadastrarUsuario(this.sessao, cpfUsuario, this.senhaFuncionario);
				} else {
					throw new SenhaInvalidaException();
				}
			} else {
				throw new RemoteException();
			}
		}catch(RemoteException e){
			this.servico = null;
			Vector<Object> parametros = new Vector<Object>();
			parametros.add(cpfUsuario);
			parametros.add(this.senhaFuncionario);
			this.gerenciadorOperacoes.inserirOperacao(new Operacao(TipoOperacao.DESCADASTRARUSUARIO, parametros));

		}
	}

	public Plano buscarPlano(String nome) throws PlanoInexistenteException, FilialNaoLogadaException{
		try{
			if(this.servico != null){
				return this.servico.buscarPlano(nome);
			} else {
				throw new FilialNaoLogadaException("local");
			}
		}catch(RemoteException e){
			this.servico = null;
			throw new FilialNaoLogadaException(this.filial);
		}
	}

	public IteratorBidirecional<Plano> getPlanos() throws PlanoInexistenteException, FilialNaoLogadaException{
		try{
			if(this.servico != null){
				return this.servico.getPlanos();
			} else {
				throw new RemoteException();
			}
		}catch(RemoteException e){
			this.servico = null;
			return this.gerenciadorPlanoOffline.getPlanos();
		}
	}

	public void inserirLocacao(long cpf, int codigoCopia, Periodo periodo, String senha) throws UsuarioNaoLogadoException, LocacaoExistenteException, LocacaoInvalidaException, UsuarioSemPermissaoException, CopiaLocadaNoPeriodoException, CopiaReservadaNoPeriodoException, OperacaoExistenteException, IOException, OperacaoInvalidaException, UsuarioInexistenteException, SenhaInvalidaException, CopiaInexistenteException{
		try{
			if(this.servico != null){
				Usuario user = this.servico.buscarUsuario(this.sessao, cpf);
				if(senha.equals(user.getSenha())){
					Copia c = this.servico.buscarCopia(codigoCopia);
					Locacao l = new Locacao((Cliente) user, c, periodo); 
					this.servico.inserirLocacao(this.sessao, l);
				} else {
					throw new SenhaInvalidaException();
				}
			} else {
				throw new RemoteException();
			}
		}catch(RemoteException e){
			this.servico = null;
			Vector<Object> parametros = new Vector<Object>();
			parametros.add(cpf);
			parametros.add(codigoCopia);
			parametros.add(periodo);
			this.gerenciadorOperacoes.inserirOperacao(new Operacao(TipoOperacao.INSERIRLOCACAO, parametros));
		}
	}

	public Locacao buscarLocacao(Copia copia) throws UsuarioNaoLogadoException, UsuarioSemPermissaoException, LocacaoInexistenteException, FilialNaoLogadaException{
		try{
			if(this.servico != null){
				return this.servico.buscarLocacao(this.sessao, copia);
			} else {
				throw new FilialNaoLogadaException(this.filial);
			}
		}catch(RemoteException e){
			this.servico = null;
			throw new FilialNaoLogadaException(this.filial);
		}
	}

	public IteratorBidirecional<Locacao> buscarLocacoes(Cliente cliente) throws LocacaoInexistenteException, UsuarioSemPermissaoException, UsuarioNaoLogadoException, FilialNaoLogadaException{
		try{
			if(this.servico != null){
				return this.servico.buscarLocacoes(this.sessao, cliente);
			} else {
				throw new FilialNaoLogadaException(this.filial);
			}
		}catch(RemoteException e){
			this.servico = null;
			throw new FilialNaoLogadaException(this.filial);
		}
	}

	public void inserirReserva(long cpf, int codigoCopia, Periodo periodo, String senha) throws UsuarioNaoLogadoException, ReservaExistenteException, UsuarioSemPermissaoException, CopiaLocadaNoPeriodoException, CopiaReservadaNoPeriodoException, FilialNaoLogadaException, ReservaInvalidaException, UsuarioInexistenteException, CopiaInexistenteException, SenhaInvalidaException{
		try{
			if(this.servico != null){
				Cliente c = (Cliente) this.servico.buscarUsuario(this.sessao, cpf);
				if(c.getSenha().equals(senha)){
					Copia copia = this.servico.buscarCopia(codigoCopia);
					this.servico.inserirReserva(this.sessao, new Reserva(c, copia, periodo));
				} else{
					throw new SenhaInvalidaException();
				}
			} else {
				throw new FilialNaoLogadaException(this.filial);
			}
		}catch(RemoteException e){
			this.servico = null;
			throw new FilialNaoLogadaException(this.filial);
		} catch (FilialInexistenteException e) {
			//NAO deve acontever
		}
	}

	public Reserva buscarReserva(Cliente cliente, Copia copia) throws UsuarioNaoLogadoException, ReservaInexistenteException, UsuarioSemPermissaoException, FilialNaoLogadaException{
		try{
			if(this.servico != null){
				return this.servico.buscarReserva(this.sessao, cliente, copia);
			} else {
				throw new FilialNaoLogadaException(this.filial);
			}
		}catch(RemoteException e){
			this.servico = null;
			throw new FilialNaoLogadaException(this.filial);
		}
	}

	public IteratorBidirecional<Reserva> buscarReservas(int copia) throws UsuarioNaoLogadoException, ReservaInexistenteException, UsuarioSemPermissaoException, FilialNaoLogadaException, CopiaInexistenteException{
		try{
			if(this.servico != null){

				return this.servico.buscarReservas(this.sessao, this.servico.buscarCopia(copia));
			} else {
				throw new FilialNaoLogadaException("local");
			}
		}catch(RemoteException e){
			this.servico = null;
			throw new FilialNaoLogadaException(this.filial);
		}
	}

	public IteratorBidirecional<Reserva> buscarReservas(long cpf) throws ReservaInexistenteException, UsuarioNaoLogadoException, UsuarioSemPermissaoException, FilialNaoLogadaException, UsuarioInexistenteException{
		try{
			if(this.servico != null){
				return this.servico.buscarReservas(this.sessao, (Cliente) this.servico.buscarUsuario(this.sessao, cpf));
			} else {
				throw new FilialNaoLogadaException("local");
			}
		}catch(RemoteException e){
			this.servico = null;
			throw new FilialNaoLogadaException(this.filial);
		}
	}

	public void removerReserva(long cpf, int codigoCopia) throws ReservaInexistenteException, UsuarioNaoLogadoException, UsuarioSemPermissaoException, FilialNaoLogadaException, UsuarioInexistenteException, CopiaInexistenteException, ProdutoInexistenteException{
		try{
			if(this.servico != null){
				this.servico.removerReserva(this.sessao, cpf, codigoCopia);
			} else {
				throw new FilialNaoLogadaException("local");
			}
		}catch(RemoteException e){
			this.servico = null;
			throw new FilialNaoLogadaException(this.filial);
		}
	}

	public Produto buscarProduto(String nomeProduto) throws ProdutoInexistenteException, FilialNaoLogadaException{
		try{
			if(this.servico != null){
				return this.servico.buscarProduto(nomeProduto);
			} else {
				throw new FilialNaoLogadaException("local");
			}
		}catch(RemoteException e){
			this.servico = null;
			throw new FilialNaoLogadaException(this.filial);
		}
	}

	public double devolverProduto(int codigoCopia, Calendar dataInicio) throws UsuarioNaoLogadoException, UsuarioSemPermissaoException, LocacaoInexistenteException, ProdutoInexistenteException, CopiaInexistenteException, FilialInvalidaException, OperacaoExistenteException, IOException, OperacaoInvalidaException, FilialInexistenteException, UsuarioInexistenteException{
		double retorno = 0;
		try{
			if(this.servico != null){
				Locacao locacao = this.servico.buscarLocacao(this.sessao, this.servico.buscarCopia(codigoCopia));
				this.servico.devolverProduto(this.sessao, codigoCopia, this.filial);

				retorno = locacao.getCusto();

				Calendar dataEntrega = Calendar.getInstance();

				if(dataEntrega.after(locacao.getPeriodo().getDataFim())){
					retorno += locacao.getPeriodo().getQuantidadeDias() * locacao.getCopia().getProduto().getClassificacaoPreco().getPreco();
				}
			} else {
				throw new RemoteException();
			}
		}catch(RemoteException e){
			this.servico = null;
			Vector<Object> parametros = new Vector<Object>();
			parametros.add(codigoCopia);
			parametros.add(this.filial);
			this.gerenciadorOperacoes.inserirOperacao(new Operacao(TipoOperacao.DEVOLVERPRODUTO, parametros));

			retorno = -1;
		}

		return retorno;
	}

	public Long getSessao() {
		return sessao;
	}

	public void setSessao(Long sessao) {
		this.sessao = sessao;
	}

	public String getNomeFilial(){
		return this.filial;
	}

	public IteratorBidirecional<Entrega> getEntregas() throws EntregaInexistenteException{
		return this.gerenciadorEntregas.getEntregas();
	}

	public void inserirEntrega(Entrega entrega){
		this.gerenciadorEntregas.inserirEntrega(entrega);

		if(this.janelaPrincipal != null){
			this.janelaPrincipal.notificarEntrega();
		}
	}

	public void removerEntrega(int codigoCopia){
		this.gerenciadorEntregas.removerEntrega(codigoCopia);
	}

	public Copia buscarCopia(int codigoCopia) throws FilialNaoLogadaException, CopiaInexistenteException{
		try{
			if(this.servico != null){
				return this.servico.buscarCopia(codigoCopia);
			} else {
				throw new FilialNaoLogadaException(this.filial);
			}
		}catch(RemoteException e){
			this.servico = null;
			throw new FilialNaoLogadaException(this.filial);
		}
	}

	public void removerOperacao(Operacao o) throws OperacaoInexistenteException, IOException{
		this.gerenciadorOperacoes.removerOperacao(o);
	}

	public void validarOperacao(Operacao op) throws ErroValidacaoException, OperacaoInexistenteException, FilialNaoLogadaException{
		Vector<Object> v = op.getParametros();

		if(this.servico != null){
			try {
				switch(op.getTipoOperacao()){
				case ALTERARDADOSUSUARIO:
					this.servico.alterarDadosUsuario(this.sessao, (Usuario) v.get(0));
					break;
				case CADASTRARCLIENTE:
					Cliente c = (Cliente) v.get(0);
					Telefone[] t = new Telefone[1];
					t[0] = (Telefone) v.get(1);
					
					c.setTelefones(t);
					this.servico.cadastrarUsuario(this.sessao, c);
					break;
				case CADASTRARDEPENDENTE:
					this.servico.cadastrarUsuario(this.sessao, new Dependente((Long)v.get(0), (String) v.get(1), (String) v.get(2), (String) v.get(3), (Cliente) this.servico.buscarUsuario(this.sessao, (Long) v.get(4)))); 
					break;
				case DESCADASTRARUSUARIO:
					this.servico.descadastrarUsuario(this.sessao, (Long) v.get(0), (String) v.get(1));
					break;
				case DEVOLVERPRODUTO:
					this.servico.devolverProduto(this.sessao, (Integer) v.get(0), (String) v.get(1));
					break;
				case INSERIRLOCACAO:
					Usuario user = this.servico.buscarUsuario(this.sessao, (Long) v.get(0));
					Copia copia = this.servico.buscarCopia((Integer) v.get(1));
					Locacao l = new Locacao((Cliente) user, copia, (Periodo) v.get(2)); 

					this.servico.inserirLocacao(this.sessao, l);
					break;
				}
			} catch(Exception e){
				throw new ErroValidacaoException(e);
			}
		} else {
			throw new FilialNaoLogadaException(this.filial);
		}
	}

	public void notificarReserva(Reserva reserva) {
		this.reservasNotificadas.add(reserva);

		if(this.janelaPrincipal != null){
			this.janelaPrincipal.notificarReserva();
		}
	}

	public void removerReservaNotificada(int codigo){
		boolean achou = false;

		for(int i = 0; i < this.reservasNotificadas.size() && !achou; i++){
			if(this.reservasNotificadas.get(i).getCopia().getCodigoCopia() == codigo){
				this.reservasNotificadas.remove(i);
				achou = true;
			}
		}
	}

	public IteratorBidirecional<Reserva> getReservasNotificadas() throws ReservaInexistenteException{
		if(this.reservasNotificadas.size() > 0){
			return new IteratorBidirecional<Reserva>(this.reservasNotificadas.iterator());
		} else throw new ReservaInexistenteException();
	}

	public void notificarRemocaoReserva(Reserva reserva) {
		boolean achou = false;
		for(int i = 0; i < this.reservasNotificadas.size() && !achou; i++){
			if(this.reservasNotificadas.get(i).getCopia().getCodigoCopia() == reserva.getCopia().getCodigoCopia()){
				this.reservasNotificadas.remove(i);
				achou = true;
			}
		}

		if(this.janelaPrincipal != null){
			this.janelaPrincipal.notificarRemocaoReserva(reserva);
		}
	}
	/*
	public static void main(String[] args) {
		try {
			Fachada fachada = Fachada.getInstancia(null);
			fachada.logar(Long.parseLong("32137915472"), "senhateste");
			fachada.logarFilial();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (NumberFormatException e) {
			e.printStackTrace();
		} catch (UsuarioLogadoException e) {
			e.printStackTrace();
		} catch (UsuarioInexistenteException e) {
			e.printStackTrace();
		} catch (SenhaInvalidaException e) {
			e.printStackTrace();
		} catch (FilialInexistenteException e) {
			e.printStackTrace();
		} catch (UsuarioNaoLogadoException e) {
			e.printStackTrace();
		} catch (FilialLogadaException e) {
			e.printStackTrace();
		} catch (FilialNaoLogadaException e) {
			e.printStackTrace();
		} catch (UsuarioSemPermissaoException e) {
			e.printStackTrace();
		}
	}
	 */
}
