package gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import dados.locacao.Reserva;

import negocio.exception.FilialInexistenteException;
import negocio.exception.FilialLogadaException;
import negocio.exception.FilialNaoLogadaException;
import negocio.exception.UsuarioNaoLogadoException;
import negocio.exception.UsuarioSemPermissaoException;

import fachada.Fachada;

public class JanelaFilial extends JFrame implements ActionListener{

	private static final long serialVersionUID = 1L;

	private JMenuItem menuCadastroCliente;
	private JMenuItem menuCadastroDependente;
	private JMenuItem menuLogarFuncionario;
	private JMenuItem menuLogarFilial;
	private JMenuItem menuLocar;
	private JMenuItem menuDevolver;
	private JMenuItem menuDescadastrarUsuario;
	private JMenuItem menuFazerReserva;
	private JMenuItem menuCancelarReserva;
	private JMenuItem menuAtualizarCliente;
	private JMenuItem menuAtualizarDependente;
	private JMenuItem menuDeslogarFuncionario;
	private JMenuItem menuDeslogarFilial;
	private JMenuItem menuOperacoesPendentes;
	private JMenuItem menuEntregasPendentes;

	private JPanel painelExibido;
	private PainelInicial painelInicial;
	private PainelClienteEDependente painelClienteDependente;
	private PainelLogin painelLogin;
	private PainelLocacao painelLocacao;
	private PainelReserva painelReserva;
	private PainelOperacoesEntregas painelOperacoesEntregas;


	public JanelaFilial(){
		super("VideoSystem");
		this.setSize(770,625);
		this.setLayout(null);

		JMenuBar barraMenu = new JMenuBar();
		this.add(barraMenu);
		barraMenu.setVisible(true);
		barraMenu.setBackground(Color.WHITE);
		barraMenu.setSize(this.getWidth(), 25);

		JMenu menuLogar = criarMenu("Login", barraMenu);
		JMenu menuCadastrar = criarMenu("Cadastro", barraMenu);
		JMenu menuLocacao = criarMenu("Locaao", barraMenu);
		JMenu menuReserva = criarMenu("Reservas", barraMenu);
		JMenu menuOperacoes = criarMenu("Operaes", barraMenu);
		JMenu menuEntregas = criarMenu("Entregas", barraMenu);
		JMenu menuLogoff = criarMenu("Logoff", barraMenu);
		menuCadastroCliente = criarMenuItem("Cadastro Cliente", menuCadastrar);
		menuCadastroDependente = criarMenuItem("Cadastro Dependente", menuCadastrar);
		menuAtualizarCliente = criarMenuItem("Atualizar Cadastro Cliente", menuCadastrar);
		menuAtualizarDependente = criarMenuItem("Atualizar Cadastro Dependente", menuCadastrar);
		menuDescadastrarUsuario = criarMenuItem("Descadastro Cliente", menuCadastrar);
		menuLogarFuncionario = criarMenuItem("Login Funcionario", menuLogar);
		menuLogarFilial = criarMenuItem("Login Filial", menuLogar);
		menuLocar = criarMenuItem("Locao Produto", menuLocacao);
		menuDevolver = criarMenuItem("Devoluo Produto", menuLocacao);
		menuFazerReserva = criarMenuItem("Fazer Reserva", menuReserva);
		menuCancelarReserva = criarMenuItem("Cancelar Reserva", menuReserva);
		menuDeslogarFuncionario = criarMenuItem("Deslogar Funcionrio", menuLogoff);
		menuDeslogarFilial = criarMenuItem("Deslogar Filial", menuLogoff);
		menuOperacoesPendentes = criarMenuItem("Operaes pendentes", menuOperacoes);
		menuEntregasPendentes = criarMenuItem("Entregas e Reservas pendentes", menuEntregas);

		menuCadastrar.insertSeparator(2); 
		menuCadastrar.insertSeparator(5);

		painelInicial = new PainelInicial(this);
		painelExibido = this.painelInicial;
		painelExibido.setVisible(true);
		painelClienteDependente = new PainelClienteEDependente(this);
		painelLogin = new PainelLogin(this);
		painelLocacao = new PainelLocacao(this);
		painelReserva = new PainelReserva(this);
		painelOperacoesEntregas = new PainelOperacoesEntregas(this);


		this.add(painelInicial);
		this.add(painelClienteDependente);
		this.add(painelLogin);
		this.add(painelLocacao);
		this.add(painelReserva);
		this.add(painelOperacoesEntregas);
		this.setResizable(false);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		this.habilitarFuncionarioLogado(false);
		this.habilitarFilialLogada(false);

		this.trocarPainel(this.painelInicial);
	}

	private JMenu criarMenu(String text, JMenuBar barra){
		JMenu retorno = new JMenu(text);
		retorno.setForeground(Color.BLACK);
		retorno.setBackground(Color.WHITE);
		barra.add(retorno);
		return retorno;
	}

	private JMenuItem criarMenuItem(String text, JMenu menu){
		JMenuItem retorno = new JMenuItem(text);
		retorno.setBackground(Color.WHITE);
		retorno.setForeground(Color.BLACK);
		menu.add(retorno);
		retorno.addActionListener(this);
		return retorno;
	}

	public void trocarPainel(JPanel p){
		this.painelExibido.setVisible(false);
		((PainelFigura) this.painelExibido ).desaparecer();
		this.painelExibido = p;
		((PainelFigura) p ).aparecer();
		this.painelExibido.setVisible(true);
	}

	public PainelInicial getPainelInicial(){
		return this.painelInicial;
	}

	public void habilitarFuncionarioLogado(boolean b){
		this.menuLogarFuncionario.setEnabled(!b);
		this.menuLogarFilial.setEnabled(b);
		this.menuDeslogarFuncionario.setEnabled(b);
		this.menuFazerReserva.setEnabled(b);
		this.menuCancelarReserva.setEnabled(b);
		this.menuAtualizarCliente.setEnabled(b);
		this.menuAtualizarDependente.setEnabled(b);
	}

	public void habilitarFilialLogada(boolean b){
		this.menuDeslogarFilial.setEnabled(b);
		this.menuLogarFilial.setEnabled(!b);
	}

	@Override
	public void actionPerformed(ActionEvent a) {
		Component c = (Component) a.getSource();

		if(c == this.menuCadastroCliente){
			painelClienteDependente.limparCampos();
			painelClienteDependente.setFuncao("Cadastrar Cliente");
			this.trocarPainel(painelClienteDependente);
		} else if(c == menuCadastroDependente){
			painelClienteDependente.limparCampos();
			painelClienteDependente.setFuncao("Cadastrar Dependente");
			this.trocarPainel(painelClienteDependente);
		}else if(c == menuAtualizarCliente){
			painelClienteDependente.limparCampos();
			painelClienteDependente.setFuncao("Atualizar Cliente");
			this.trocarPainel(painelClienteDependente);
		}else if(c == menuAtualizarDependente){
			painelClienteDependente.limparCampos();
			painelClienteDependente.setFuncao("Atualizar Dependente");
			this.trocarPainel(painelClienteDependente);
		}else if(c == menuDescadastrarUsuario){
			painelClienteDependente.limparCampos();
			painelClienteDependente.setFuncao("Descadastrar Usurio");
			this.trocarPainel(painelClienteDependente);
		}else if(c == menuLogarFuncionario){
			painelLogin.limpar();
			this.trocarPainel(painelLogin);
		}else if(c == menuLocar){
			painelLocacao.limparCampos();
			painelLocacao.setFuncao("Locao Produto");
			this.trocarPainel(painelLocacao);
		}else if(c == menuDevolver){
			painelLocacao.limparCampos();
			painelLocacao.setFuncao("Devoluo Produto");
			this.trocarPainel(painelLocacao);
		}else if(c == menuFazerReserva){
			painelReserva.limpar();
			painelReserva.setFuncao("Fazer Reserva");
			this.trocarPainel(painelReserva);
		}else if(c == menuCancelarReserva){
			painelReserva.limpar();
			painelReserva.setFuncao("Cancelar Reserva");
			this.trocarPainel(painelReserva);
		}else if(c == menuOperacoesPendentes){
			this.painelOperacoesEntregas.carregarOperacoes();
			painelOperacoesEntregas.setFuncao("Operaes Pendentes");
			this.trocarPainel(painelOperacoesEntregas);
		}else if(c == menuEntregasPendentes){
			this.painelOperacoesEntregas.carregarEntregas();
			this.painelOperacoesEntregas.carregarReservas();
			painelOperacoesEntregas.setFuncao("Entregas Pendentes");
			this.trocarPainel(painelOperacoesEntregas);
		} else if(c == menuLogarFilial){
			try {
				Fachada.getInstancia(this).logarFilial();
				
				this.habilitarFilialLogada(true);

				if(Fachada.getInstancia(this).existeOperacao()){
					JOptionPane.showMessageDialog(this, "Filial logada.\nO sistema ir validar as operaes pendentes.", "Informao", JOptionPane.INFORMATION_MESSAGE);
					this.painelOperacoesEntregas.carregarOperacoes();
					this.painelOperacoesEntregas.setFuncao("Operaes Pendentes");
					this.trocarPainel(this.painelOperacoesEntregas);
					if(!this.painelOperacoesEntregas.validarOperacoes()){
						JOptionPane.showMessageDialog(this, "Algumas operaes no puderam ser validadas. Verifique o arquivo 'erro validao.txt' para maiores informaes", "Erro!", JOptionPane.ERROR_MESSAGE);
					} else {
						this.painelOperacoesEntregas.carregarOperacoes();
						JOptionPane.showMessageDialog(this, "As operaes foram validadas com sucesso!", "Informao!", JOptionPane.INFORMATION_MESSAGE);
					}
				} else {
					JOptionPane.showMessageDialog(this, "Filial logada com sucesso.", "Informao", JOptionPane.INFORMATION_MESSAGE);
				}
			} catch (UsuarioNaoLogadoException e) {
				JOptionPane.showMessageDialog(this, "Para logar filial,  necessrio que o usurio esteja logado", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (FilialLogadaException e) {
				JOptionPane.showMessageDialog(this, "A filial j est logada", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (FilialInexistenteException e) {
				JOptionPane.showMessageDialog(this, "A filial no existe", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (FilialNaoLogadaException e) {
				JOptionPane.showMessageDialog(this, "A filial no est logada", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (IOException e) {
				JOptionPane.showMessageDialog(this, "Arquivo com informao da filial no pode ser lido", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (UsuarioSemPermissaoException e) {
				JOptionPane.showMessageDialog(this, "O usurio logado no tem permisso para logar esta filial.\nVerifique o nome da filial existente no arquivo de configurao.", "Erro!", JOptionPane.ERROR_MESSAGE);
			}
		} else if(c == this.menuDeslogarFuncionario){
			try {
				Fachada.getInstancia(this).deslogar();
				this.habilitarFuncionarioLogado(false);
				this.habilitarFilialLogada(false);
				this.trocarPainel(this.painelInicial);
				JOptionPane.showMessageDialog(this, "Funcionrio deslogado com sucesso.", "Informao", JOptionPane.INFORMATION_MESSAGE);
			} catch (UsuarioNaoLogadoException e) {
				JOptionPane.showMessageDialog(this, "O usurio no est logado", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (FilialNaoLogadaException e) {
				JOptionPane.showMessageDialog(this, "A filial no est logada", "Erro!", JOptionPane.ERROR_MESSAGE);
			}
		} else if(c == this.menuDeslogarFilial){
			try {
				Fachada.getInstancia(this).deslogarFilial();
				this.habilitarFilialLogada(false);
				this.trocarPainel(this.painelInicial);
			} catch (FilialNaoLogadaException e) {
				JOptionPane.showMessageDialog(this, "A filial no est logada", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (FilialInexistenteException e) {
				JOptionPane.showMessageDialog(this, "A Filial no existe", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (UsuarioNaoLogadoException e) {
				JOptionPane.showMessageDialog(this, "Para deslogar uma filial,  necessrio que o usurio esteja logado", "Erro!", JOptionPane.ERROR_MESSAGE);

			}
		}
	} 

	public void notificarEntrega() {
		this.painelOperacoesEntregas.carregarEntregas();
		JOptionPane.showMessageDialog(this, "Novo pedido de entrega recebido.\nVeja mais detalhes no menu 'Entregas'", "Informao", JOptionPane.INFORMATION_MESSAGE);
	}

	public void notificarReserva() {
		this.painelOperacoesEntregas.carregarReservas();
		JOptionPane.showMessageDialog(this, "Novo pedido de reserva recebido.\nVeja mais detalhes no menu 'Entregas'", "Informao", JOptionPane.INFORMATION_MESSAGE);
	}

	public void notificarRemocaoReserva(Reserva reserva) {
		this.painelOperacoesEntregas.carregarReservas();
		JOptionPane.showMessageDialog(this, "Reserva cancelada pelo cliente:\n" + reserva.toString(), "Informao", JOptionPane.INFORMATION_MESSAGE);
	}

	public static void main(String [] args){
		new JanelaFilial().setVisible(true);
	}

}