package gui;


import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.io.IOException;
import java.util.Iterator;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import util.IteratorBidirecional;

import negocio.exception.CampoInvalidoException;
import negocio.exception.FilialNaoLogadaException;
import negocio.exception.OperacaoExistenteException;
import negocio.exception.PlanoInexistenteException;
import negocio.exception.SenhaInvalidaException;
import negocio.exception.UsuarioExistenteException;
import negocio.exception.UsuarioInexistenteException;
import negocio.exception.UsuarioNaoLogadoException;
import negocio.exception.UsuarioSemPermissaoException;

import dados.exception.CpfInvalidoException;
import dados.exception.EmailInvalidoException;
import dados.exception.EnderecoInvalidoException;
import dados.exception.NomeInvalidoException;
import dados.exception.OperacaoInvalidaException;
import dados.exception.PlanoInvalidoException;
import dados.exception.TelefoneInvalidoException;
import dados.filial.Endereco;
import dados.usuario.Cliente;
import dados.usuario.Dependente;
import dados.usuario.Plano;
import dados.usuario.Telefone;
import dados.usuario.Usuario;
import fachada.Fachada;


public class PainelClienteEDependente extends PainelFigura {

	private static final long serialVersionUID = 1L;

	private String funcao;

	private JLabel labelTitulo;
	private JLabel labelNome;
	private JLabel labelTelefone;
	private JLabel labelCPF;
	private JLabel labelCPFCliente;
	private JLabel labelEmail;
	private JLabel labelPlanos;

	private JTextField textoNome;
	private JTextField textoDDD;
	private JTextField textoTelefone;
	private JTextField textoCPF;
	private JTextField textoCPFCliente;
	private JTextField textoEmail;

	private JComboBox listaPlanos;

	private JButton botao;
	private JButton botaoRemover;
	private JButton botaoBuscar;

	private PainelEndereco painelEndereco;
	private PainelSenha painelSenha;
	private Usuario usuario;

	public PainelClienteEDependente(JanelaFilial janela){
		super("images/telaSecundaria.png", janela);

		this.funcao = "";

		labelTitulo = criarRotulo("Cadastrar Cliente", 300, 60, 300, 40);
		labelTitulo.setFont(new Font("Arial",Font.BOLD ,20));

		labelNome = criarRotulo("Nome:", 20, 200, 100, 30);
		labelCPF = criarRotulo("CPF:", 20, 150, 250, 30);
		textoNome = criarCampo(70, 200, 250, 25);
		textoCPF = criarCampo(60, 150, 150, 25);
		labelTelefone = criarRotulo("Telefone:", 20, 250, 150, 30);
		labelEmail = criarRotulo("E-mail:", 20, 300, 100, 30);
		labelCPFCliente = criarRotulo("CPF do cliente:", 320, 350, 150, 30);
		labelPlanos = criarRotulo("Planos:", 20, 350, 150, 30);


		textoDDD =  criarCampo(90, 250, 30, 25);
		textoTelefone = criarCampo(130, 250, 100, 25);
		textoEmail = criarCampo(70, 300, 250, 25);
		textoCPFCliente =  criarCampo(430, 350, 150, 25); 
		listaPlanos = criarComboBox(100, 350, 100, 25);

		botao = criarBotao("Cadastrar", 330, 500, 120, 40);
		botaoRemover = criarBotao("Remover", 410, 500, 120, 40);
		botaoRemover.setEnabled(false);

		botaoBuscar = criarBotao("Buscar", 220, 150, 70, 25);

		labelTitulo.setVisible(true);
		labelCPF.setVisible(true);
		labelNome.setVisible(true);
		labelTelefone.setVisible(true);
		labelCPFCliente.setVisible(false);
		labelPlanos.setVisible(true);
		textoTelefone.setVisible(true);
		textoCPF.setVisible(true);
		textoDDD.setVisible(true);
		textoNome.setVisible(true);
		textoCPFCliente.setVisible(false);
		listaPlanos.setVisible(true);

		painelEndereco = new PainelEndereco(this.janelaPrincipal);
		this.add(painelEndereco);
		painelEndereco.setLocation(345,130);
		painelEndereco.setSize(400, 250);
		painelEndereco.setVisible(true);

		painelSenha = new PainelSenha(this.janelaPrincipal);
		this.add(painelSenha);
		painelSenha.setLocation(20,400);
		painelSenha.setVisible(true);

		this.listaPlanos.addActionListener(this);
	}

	public void mostrarCamposExtras(boolean mostrar){
		labelCPFCliente.setVisible(mostrar);
		textoCPFCliente.setVisible(mostrar);
	}

	public void mostrarCamposComunsCadastro(boolean mostrar){
		labelNome.setVisible(mostrar);
		labelTelefone.setVisible(mostrar);
		labelEmail.setVisible(mostrar);
		textoTelefone.setVisible(mostrar);
		textoDDD.setVisible(mostrar);
		textoNome.setVisible(mostrar);
		textoEmail.setVisible(mostrar);
	}

	public void modificarPosicoes(boolean modificar){

		if(!modificar){
			labelNome.setLocation(20, 200);
			labelCPF.setLocation(20, 150);
			textoCPF.setLocation(60, 150);
			textoNome.setLocation(70, 200);
			labelTelefone.setLocation(20, 250);
			labelEmail.setLocation(20, 300);
			textoDDD.setLocation(90, 250);
			textoTelefone.setLocation(130, 250);
			textoEmail.setLocation(70, 300);
		}else{
			labelCPF.setLocation(320, 150);
			labelNome.setLocation(320, 200);
			labelTelefone.setLocation(320, 250);
			labelEmail.setLocation(320, 300);
			textoNome.setLocation(370, 200);
			textoCPF.setLocation(360, 150);
			textoDDD.setLocation(390, 250);
			textoTelefone.setLocation(430, 250);
			textoEmail.setLocation(370, 300);
			labelTelefone.setVisible(false);
			textoDDD.setVisible(false);
			textoTelefone.setVisible(false);
		}

	}

	public void habilitarCampos(boolean habilitar){
		textoTelefone.setEditable(habilitar);
		textoDDD.setEditable(habilitar);
		textoNome.setEditable(habilitar);
		textoCPFCliente.setEditable(habilitar);
		listaPlanos.setEnabled(habilitar);
		textoEmail.setEditable(habilitar);
		painelSenha.setVisible(habilitar);
		botaoBuscar.setVisible(!habilitar);
	}

	public void limparCampos(){
		textoCPF.setText("");
		textoCPFCliente.setText("");
		textoDDD.setText("");
		textoEmail.setText("");
		textoNome.setText("");
		textoTelefone.setText("");
		painelSenha.textoSenha.setText("");

		this.listaPlanos.removeAllItems();

		try {
			Iterator<Plano> it = Fachada.getInstancia(this.janelaPrincipal).getPlanos();

			while(it.hasNext()){
				Plano p = it.next();
				this.listaPlanos.addItem(p.getNome());
			} 
		}catch (Exception e) {
		}
	}

	public void setFuncao(String funcao){

		boolean mostrarCamposExtras = false;
		boolean mostrarCamposComunsCadastrar = true;
		boolean mostrarPlanos = false;
		boolean habilitarCampos = true;
		painelEndereco.setVisible(false);
		String titulo = "Cadastrar Cliente";
		String tituloBotao = "";
		labelCPF.setLocation(20, 150);
		textoCPF.setLocation(60, 150);
		botao.setLocation(335, 500);
		labelTitulo.setLocation(300, 60);
		boolean modificarPosicoes = false;
		labelCPF.setText("CPF:");

		if(!funcao.isEmpty()){
			if(funcao.equalsIgnoreCase("Cadastrar Cliente")){
				mostrarCamposExtras = false;
				mostrarCamposComunsCadastrar = true;
				painelEndereco.setVisible(true);
				mostrarPlanos = true;
				tituloBotao = "Cadastrar";
			}else if(funcao.equalsIgnoreCase("Cadastrar Dependente")){
				titulo = "Cadastrar Dependente";
				mostrarCamposExtras = true;
				mostrarCamposComunsCadastrar = true;
				labelTitulo.setLocation(290, 60);
				modificarPosicoes = true;
				tituloBotao = "Cadastrar";
			}else if(funcao.equalsIgnoreCase("Atualizar Cliente")){
				titulo = "Atualizar Cliente";

				botao.setEnabled(false);
				botaoRemover.setEnabled(false);

				mostrarCamposExtras = false;
				mostrarCamposComunsCadastrar = true;
				painelEndereco.setVisible(true);
				habilitarCampos = false;
				mostrarPlanos = true;
				tituloBotao = "Atualizar";
				botao.setLocation(230, 500);
				botaoBuscar.setLocation(230, 150);
			}else if(funcao.equalsIgnoreCase("Atualizar Dependente")){
				titulo = "Atualizar Dependente";

				botao.setEnabled(false);
				botaoRemover.setEnabled(false);

				mostrarCamposExtras = true;
				mostrarCamposComunsCadastrar = true;
				labelTitulo.setLocation(290, 60);
				modificarPosicoes = true;
				tituloBotao = "Cadastrar";
				habilitarCampos = false;
				botao.setLocation(230, 500);
				botaoBuscar.setLocation(530, 150);
			}else if(funcao.equalsIgnoreCase("Descadastrar Usurio")){
				titulo = "Descadastrar Cliente";
				mostrarCamposComunsCadastrar = false;
				mostrarCamposExtras = false;
				tituloBotao = "Descadastrar";
				botao.setLocation(335, 400);
				labelCPF.setText("CPF do Cliente:");
			}
		}
		mostrarCamposExtras(mostrarCamposExtras);
		mostrarCamposComunsCadastro(mostrarCamposComunsCadastrar);
		habilitarCampos(habilitarCampos);
		modificarPosicoes(modificarPosicoes);
		painelEndereco.habilitarCamposEndereco(habilitarCampos);
		labelPlanos.setVisible(mostrarPlanos);
		listaPlanos.setVisible(mostrarPlanos);
		labelTitulo.setText(titulo);
		botao.setText(tituloBotao);
		botaoRemover.setVisible(!habilitarCampos);
		this.funcao = funcao;

		if(funcao.equalsIgnoreCase("Descadastrar Usurio")){
			labelCPF.setLocation(345, 200);
			textoCPF.setLocation(325, 230);
		}

	}

	private Cliente lerCliente() throws EnderecoInvalidoException, PlanoInexistenteException, FilialNaoLogadaException, NumberFormatException, NomeInvalidoException, SenhaInvalidaException, CpfInvalidoException, PlanoInvalidoException, EmailInvalidoException, CampoInvalidoException {
		String cpfString = textoCPF.getText();
		String nome = textoNome.getText();
		String email = textoEmail.getText();
		String planoString = (String)listaPlanos.getSelectedItem();
		String senha = painelSenha.getSenha();

		Plano tipoPlano = null;
		Endereco endereco = painelEndereco.getEndereco();
		Iterator<Plano> iteratorPlano = Fachada.getInstancia(this.janelaPrincipal).getPlanos();
		if(iteratorPlano != null){
			while(iteratorPlano.hasNext()){
				Plano plano = iteratorPlano.next();
				if(!planoString.isEmpty()){
					if(plano.getNome().equalsIgnoreCase(planoString)){
						tipoPlano = plano;
					}
				}
			}
		}

		Cliente user = null;

		if(!cpfString.isEmpty() && !nome.isEmpty() && !email.isEmpty() && tipoPlano != null && endereco != null && !senha.isEmpty()){	
			user = new Cliente(Long.parseLong(cpfString),senha, nome, email, tipoPlano, endereco);
		} else {
			throw new CampoInvalidoException();
		}
		return user;
	}


	@Override
	public void actionPerformed(ActionEvent a) {
		Component c = (Component) a.getSource();
		if(!funcao.isEmpty()){
			try {
				Fachada f = Fachada.getInstancia(this.janelaPrincipal);
				if(c == botao){
					if(this.funcao.equalsIgnoreCase("Cadastrar Cliente")){
						String ddd = textoDDD.getText();
						String numeroTelefone = textoTelefone.getText();
						if(!ddd.isEmpty() && !numeroTelefone.isEmpty()){
							Telefone t = new Telefone(Integer.parseInt(ddd), Integer.parseInt(numeroTelefone));
							f.cadastrarCliente(this.lerCliente(), t);
							JOptionPane.showMessageDialog(this, "Cliente cadastrado com sucesso", "Informao", JOptionPane.INFORMATION_MESSAGE);

						}else{
							throw new CampoInvalidoException();
						}
					}else if(this.funcao.equalsIgnoreCase("Cadastrar Dependente")){
						String cpfString = textoCPF.getText();
						String nome = textoNome.getText();
						String email = textoEmail.getText();
						String cpfDonoCliente = textoCPFCliente.getText();
						String senha = painelSenha.getSenha();

						if(!cpfString.isEmpty() && !nome.isEmpty() && !email.isEmpty() && !cpfDonoCliente.isEmpty() && !senha.isEmpty()){
							long cpf = Long.parseLong(cpfString);
							f.cadastrarDependente(cpf, senha, nome, email, Long.parseLong(cpfDonoCliente));
							JOptionPane.showMessageDialog(this, "Dependente cadastrado com sucesso", "Informao", JOptionPane.INFORMATION_MESSAGE);
						}else{
							throw new CampoInvalidoException();
						}
					}else if(this.funcao.equalsIgnoreCase("Atualizar Cliente")){
						f.alterarDadosUsuario(this.lerCliente());
						JOptionPane.showMessageDialog(this, "Dados do usurio alterados com sucesso", "Informao", JOptionPane.INFORMATION_MESSAGE);
					}else if(this.funcao.equalsIgnoreCase("Atualizar Dependente")){
						String nome = textoNome.getText();
						String email = textoEmail.getText();
						String senha = painelSenha.getSenha();

						if(!nome.isEmpty() && !email.isEmpty() && !senha.isEmpty()){
							this.usuario.setNome(nome);
							this.usuario.setEmail(email);
							this.usuario.setSenha(senha);

							f.alterarDadosUsuario(this.usuario);
							JOptionPane.showMessageDialog(this, "Dados do usurio alterados com sucesso", "Informao", JOptionPane.INFORMATION_MESSAGE);
						}else{
							throw new CampoInvalidoException();
						}
					}else if(this.funcao.equalsIgnoreCase("Descadastrar Usurio")){
						String cpfString = textoCPF.getText();
						String senha = painelSenha.getSenha();
						if(!cpfString.isEmpty() && !senha.isEmpty()){
							long cpf = Long.parseLong(cpfString);
							f.descadastrarUsuario(cpf, senha);
							JOptionPane.showMessageDialog(this, "Cadastro de usurio removido com sucesso", "Informao", JOptionPane.INFORMATION_MESSAGE);
						}else{
							throw new CampoInvalidoException();
						}
					}
					this.limparCampos();
					this.painelEndereco.limpar();
				}else if(c == botaoRemover){
					if(funcao.equalsIgnoreCase("Atualizar Cliente")){
						String cpfString = textoCPF.getText();
						String senha = painelSenha.getSenha();
						if(!cpfString.isEmpty() && !senha.isEmpty()){
							f.descadastrarUsuario(Long.parseLong(cpfString), senha);
							JOptionPane.showMessageDialog(this, "Cadastro de usurio removido com sucesso", "Informao", JOptionPane.INFORMATION_MESSAGE);
						}
					}else if(funcao.equalsIgnoreCase("Atualizar Dependente")){
						String cpfString = textoCPF.getText();
						String senha = painelSenha.getSenha();
						if(!cpfString.isEmpty() && !senha.isEmpty()){
							f.descadastrarUsuario(Long.parseLong(cpfString), senha);
							JOptionPane.showMessageDialog(this, "Cadastro de usurio removido com sucesso", "Informao", JOptionPane.INFORMATION_MESSAGE);
						}
					}
					this.limparCampos();
					this.painelEndereco.limpar();
				}else if(c == botaoBuscar){
					if(funcao.equalsIgnoreCase("Atualizar Cliente")){
						String cpfString = textoCPF.getText();
						botaoRemover.setEnabled(true);
						botao.setEnabled(true);

						if(!cpfString.isEmpty()){
							long cpf = Long.parseLong(cpfString);
							this.usuario = f.buscarUsuario(cpf);
							textoNome.setText(this.usuario.getNome());
							textoEmail.setText(this.usuario.getEmail());

							IteratorBidirecional<Telefone> it = ((Cliente)this.usuario).getTelefones();
							if(it.hasNext()){
								Telefone t = it.next();
								textoDDD.setText(""+t.getDDD());
								textoTelefone.setText(""+t.getNumero());	
							}
							painelEndereco.setEndereco((Cliente)this.usuario);
							this.listaPlanos.setSelectedItem(((Cliente)this.usuario).getPlano().getNome());
							habilitarCampos(true);
							textoCPF.setEditable(false);
						}
					}else if(funcao.equalsIgnoreCase("Atualizar Dependente")){
						String cpfString = textoCPF.getText();
						botaoRemover.setEnabled(true);
						botao.setEnabled(true);

						if(!cpfString.isEmpty()){
							long cpf = Long.parseLong(cpfString);
							this.usuario = f.buscarUsuario(cpf);
							textoNome.setText(this.usuario.getNome());
							textoEmail.setText(this.usuario.getEmail());
							long cpfDonoConta = ((Dependente)this.usuario).getCliente().getCpf();
							textoCPFCliente.setText(""+cpfDonoConta);
							habilitarCampos(true);
							textoCPF.setEditable(false);
							textoCPFCliente.setEditable(false);
						}
					}
				}

				//TODO mensagem de sucesso

				this.limparCampos();
				this.painelEndereco.limpar();
				this.janelaPrincipal.trocarPainel(this.janelaPrincipal.getPainelInicial());
			} catch (IOException e) {
				// TODO Auto-generated catch block
			} catch (NumberFormatException e) {
				JOptionPane.showMessageDialog(this, "Preencha o campo com nmeros", "Ateno", JOptionPane.ERROR_MESSAGE);
			} catch (NomeInvalidoException e) {
				JOptionPane.showMessageDialog(this, "O campo nome foi preechido de forma incorreta", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (SenhaInvalidaException e) {
				JOptionPane.showMessageDialog(this, "Senha invlida", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (CpfInvalidoException e) {
				JOptionPane.showMessageDialog(this, "CPF invlido", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (EnderecoInvalidoException e) {
				JOptionPane.showMessageDialog(this, "Endereo invlido", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (PlanoInvalidoException e) {
				JOptionPane.showMessageDialog(this, "O plano selecionado no  vlido", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (EmailInvalidoException e) {
				JOptionPane.showMessageDialog(this, "E-mail invlido", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (PlanoInexistenteException e) {
				JOptionPane.showMessageDialog(this, "O plano selecionado no existe", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (FilialNaoLogadaException e) {
				JOptionPane.showMessageDialog(this, "A filial no est logada", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (UsuarioSemPermissaoException e) {
				JOptionPane.showMessageDialog(this, "Usurio sem permisso para fazer esta operaa", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (UsuarioExistenteException e) {
				JOptionPane.showMessageDialog(this, "O usurio j existe", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (UsuarioNaoLogadoException e) {
				JOptionPane.showMessageDialog(this, "O usurio no est logado", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (UsuarioInexistenteException e) {
				JOptionPane.showMessageDialog(this, "O usurio no existe", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (OperacaoExistenteException e) {
				JOptionPane.showMessageDialog(this, "Operao j existente", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (OperacaoInvalidaException e) {
				JOptionPane.showMessageDialog(this, "Operao invlida", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (TelefoneInvalidoException e) {
				JOptionPane.showMessageDialog(this, "Telefone invlido", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (CampoInvalidoException e) {
				JOptionPane.showMessageDialog(this, "Preencha todos os campos", "Erro!", JOptionPane.ERROR_MESSAGE);
			}
		}
	}
}
