package gui;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionListener;
import java.util.Calendar;

import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;

public abstract class PainelFigura extends JPanel implements ActionListener {

	private static final long serialVersionUID = 1L;

	protected String imagem;
	protected JanelaFilial janelaPrincipal;
	private JLabel label;

	public PainelFigura(String imagem, JanelaFilial janela) {
		super(null);
		this.janelaPrincipal = janela;
		this.imagem = imagem;
		this.setSize(800,600);
		this.setVisible(false);
	}

	public void aparecer(){
		ImageIcon i = new ImageIcon(imagem);
		label = new JLabel(i);
		label.setSize(i.getIconWidth(), i.getIconHeight());
		this.add(label);
		label.setLocation(-30, 0);
	}

	public void desaparecer(){
		if(label != null){
			this.remove(label);
		}
	}

	protected JButton criarBotao(String text, int x, int y, int width, int height){
		JButton retorno = new JButton(text);
		retorno.setBackground(Color.WHITE);
		retorno.setForeground(Color.BLACK);
		retorno.setLocation(x, y);
		retorno.setSize(width, height);
		retorno.addActionListener(this);
		retorno.setBorder(BorderFactory.createMatteBorder(2, 2, 2, 2, Color.LIGHT_GRAY));
		this.add(retorno);

		return retorno;
	}

	protected JLabel criarRotulo(String text, int x, int y, int width, int height){
		JLabel retorno = new JLabel(text);
		retorno.setSize(width, height);
		retorno.setLocation(x, y);
		retorno.setForeground(Color.white);
		retorno.setFont(new Font("Arial", Font.BOLD, 14));
		this.add(retorno);

		return retorno;
	}

	protected JTextField criarCampo(int x, int y, int width, int height){
		JTextField retorno = new JTextField();
		retorno.setSize(width, height);
		retorno.setLocation(x, y);
		this.add(retorno);

		return retorno;
	}

	protected JComboBox criarComboBox(int x, int y, int width, int height){
		JComboBox retorno = new JComboBox(new DefaultComboBoxModel());
		retorno.setSize(width, height);
		retorno.setLocation(x, y);
		retorno.setBackground(Color.white);
		this.add(retorno);

		return retorno;
	}
	
	protected JList criarLista(int x, int y, int width, int height){
		JList retorno = new JList();
		retorno.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		retorno.setModel(new DefaultListModel());
		retorno.setLocation(x, y);
		retorno.setSize(width, height);

		return retorno;
	}

	protected JScrollPane criarBarraRolagem(JList lista){
		JScrollPane retorno = new JScrollPane(lista);
		retorno.setLocation(lista.getX(), lista.getY());
		retorno.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		retorno.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		retorno.setSize(lista.getWidth(), lista.getHeight());
		this.add(retorno);

		return retorno;
	}
	
	
	protected void criarData(JComboBox listaDia, JComboBox listaMes, JComboBox listaAno){
		for(int i = 1; i <= 31; i++){
			listaDia.addItem(i);
		}
		for(int i = 1; i <=12; i++){
			listaMes.addItem(i);
		}

		Calendar c = Calendar.getInstance();

		for(int i = 0; i < 5; i++){
			listaAno.addItem(c.get(Calendar.YEAR) + i);
		}
	}
}
