package gui;
import java.awt.Component;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.io.IOException;
import java.rmi.RemoteException;
import java.util.Calendar;
import java.util.GregorianCalendar;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import negocio.exception.CampoInvalidoException;
import negocio.exception.CopiaInexistenteException;
import negocio.exception.CopiaLocadaNoPeriodoException;
import negocio.exception.CopiaReservadaNoPeriodoException;
import negocio.exception.FilialInexistenteException;
import negocio.exception.LocacaoExistenteException;
import negocio.exception.LocacaoInexistenteException;
import negocio.exception.OperacaoExistenteException;
import negocio.exception.ProdutoInexistenteException;
import negocio.exception.SenhaInvalidaException;
import negocio.exception.UsuarioInexistenteException;
import negocio.exception.UsuarioNaoLogadoException;
import negocio.exception.UsuarioSemPermissaoException;

import dados.exception.FilialInvalidaException;
import dados.exception.LocacaoInvalidaException;
import dados.exception.OperacaoInvalidaException;
import dados.exception.PeriodoInvalidoException;
import dados.locacao.Periodo;

import fachada.Fachada;


public class PainelLocacao extends PainelFigura {

	private static final long serialVersionUID = 1L;

	private String funcao;

	private JLabel labelLocarProduto;
	private JLabel labelCodigoCopia;
	private JLabel labelCPFCliente;
	private JLabel labelDataLocacao;
	private JLabel labelDataDevolucao;
	private JLabel labelNomeDia1;
	private JLabel labelNomeMes1;
	private JLabel labelNomeAno1;
	private JLabel labelNomeDia2;
	private JLabel labelNomeMes2;
	private JLabel labelNomeAno2;

	private JComboBox listaDia1;
	private JComboBox listaMes1;
	private JComboBox listaAno1;
	private JComboBox listaDia2;
	private JComboBox listaMes2;
	private JComboBox listaAno2;

	private JTextField textoCodigoCopia;
	private JTextField textoCPFCliente;
	private JButton botao;

	private PainelSenha painelSenha;


	public PainelLocacao(JanelaFilial janela){
		super("images/telaSecundaria.png", janela);

		this.funcao = "";

		labelLocarProduto = criarRotulo("Locar Produto", 300, 60, 200, 50);
		labelLocarProduto.setFont(new Font("Arial",Font.BOLD ,20));
		labelCodigoCopia = criarRotulo("Cdigo Cpia:", 40, 150, 150, 30);
		textoCodigoCopia = criarCampo(145, 150, 100, 25);
		labelCPFCliente = criarRotulo("CPF do Cliente:", 40, 200, 200, 30);
		textoCPFCliente = criarCampo(160, 200, 100, 25);
		botao = criarBotao("", 315, 500, 120, 40);

		labelDataLocacao = criarRotulo("Data de Locao:", 400, 150, 200, 30);
		labelDataDevolucao = criarRotulo("Data de Devoluo:", 400, 250, 200, 30);

		labelNomeDia1 = criarRotulo("Dia", 535, 167, 50, 30);
		labelNomeMes1 = criarRotulo("Ms", 590, 167, 50, 30);
		labelNomeAno1 = criarRotulo("Ano", 655, 167, 50, 30);
		labelNomeDia2 = criarRotulo("Dia", 550, 267, 50, 30);
		labelNomeMes2 = criarRotulo("Ms", 610, 267, 50, 30);
		labelNomeAno2 = criarRotulo("Ano", 670, 267, 50, 30);

		listaDia1 = criarComboBox(525, 150, 50, 25);
		listaMes1 = criarComboBox(580, 150, 50, 25);
		listaAno1 = criarComboBox(635, 150, 70, 25);
		listaDia2 = criarComboBox(540, 250, 50, 25);
		listaMes2 = criarComboBox(595, 250, 50, 25);
		listaAno2 = criarComboBox(650, 250, 70, 25);

		criarData(listaDia1, listaMes1, listaAno1);
		criarData(listaDia2, listaMes2, listaAno2);

		labelNomeDia1.setFont(new Font("Arial",Font.PLAIN ,11));
		labelNomeMes1.setFont(new Font("Arial",Font.PLAIN ,11));
		labelNomeAno1.setFont(new Font("Arial",Font.PLAIN ,11));
		labelNomeDia2.setFont(new Font("Arial",Font.PLAIN ,11));
		labelNomeMes2.setFont(new Font("Arial",Font.PLAIN ,11));
		labelNomeAno2.setFont(new Font("Arial",Font.PLAIN ,11));

		painelSenha = new PainelSenha(this.janelaPrincipal);
		this.add(painelSenha);
		painelSenha.setLocation(20,400);
		painelSenha.setVisible(true);

	}

	public void mostrarTodosAtributos(boolean mostrar){
		labelDataLocacao.setVisible(mostrar);
		labelNomeDia1.setVisible(mostrar);
		labelNomeMes1.setVisible(mostrar);
		labelNomeAno1.setVisible(mostrar);
		listaDia1.setVisible(mostrar);
		listaMes1.setVisible(mostrar);
		listaAno1.setVisible(mostrar);
		labelDataDevolucao.setVisible(mostrar);
		labelNomeDia2.setVisible(mostrar);
		labelNomeMes2.setVisible(mostrar);
		labelNomeAno2.setVisible(mostrar);
		listaDia2.setVisible(mostrar);
		listaMes2.setVisible(mostrar);
		listaAno2.setVisible(mostrar);
		labelCPFCliente.setVisible(mostrar);
		textoCPFCliente.setVisible(mostrar);
	}

	public void limparCampos(){
		textoCodigoCopia.setText("");
		textoCPFCliente.setText("");
		painelSenha.textoSenha.setText("");
		this.listaAno1.setSelectedIndex(0);
		this.listaAno2.setSelectedIndex(0);
		this.listaDia1.setSelectedIndex(0);
		this.listaDia2.setSelectedIndex(0);
		this.listaMes1.setSelectedIndex(0);
		this.listaMes2.setSelectedIndex(0);
	}

	public void setFuncao(String funcao){

		String titulo = "";
		String nomeBotao = "";
		boolean mostrarTodosAtributos = false;
		labelCodigoCopia.setLocation(40, 150);
		textoCodigoCopia.setLocation(145, 150);

		if(!funcao.isEmpty()){
			if(funcao.equalsIgnoreCase("Locao Produto")){
				titulo = "Locar Produto";
				labelLocarProduto.setLocation(310,60);
				nomeBotao = "Locar";
				mostrarTodosAtributos = true;
				painelSenha.setVisible(true);
			}else if(funcao.equalsIgnoreCase("Devoluo Produto")){
				titulo = "Devoluo Produto";
				labelLocarProduto.setLocation(280,60);
				nomeBotao = "Devolver";
				painelSenha.setVisible(false);
				labelCodigoCopia.setLocation(315, 200);
				textoCodigoCopia.setLocation(315, 240);
			}
		}
		labelLocarProduto.setText(titulo);
		botao.setText(nomeBotao);
		this.mostrarTodosAtributos(mostrarTodosAtributos);
		this.funcao = funcao;
	}

	@Override
	public void actionPerformed(ActionEvent a) {
		Component c = (Component) a.getSource();

		if(!funcao.isEmpty()){
			try{
				Fachada f = Fachada.getInstancia(this.janelaPrincipal);
				if(c == botao){
					if(funcao.equalsIgnoreCase("Locao Produto")){
						String codigoCopia = textoCodigoCopia.getText();
						String cpfCliente = textoCPFCliente.getText();
						String senha = painelSenha.getSenha();

						Object diaLocacao = listaDia1.getSelectedItem();
						int diaL = Integer.parseInt(diaLocacao.toString());

						Object mesLocacao = listaMes1.getSelectedItem();
						int mesL = Integer.parseInt(mesLocacao.toString());

						Object anoLocacao = listaAno1.getSelectedItem();
						int anoL = Integer.parseInt(anoLocacao.toString());

						Object diaDevolucao = listaDia2.getSelectedItem();
						int diaD = Integer.parseInt(diaDevolucao.toString());

						Object mesDevolucao = listaMes2.getSelectedItem();
						int mesD = Integer.parseInt(mesDevolucao.toString());

						Object anoDevolucao = listaAno2.getSelectedItem();
						int anoD = Integer.parseInt(anoDevolucao.toString());

						GregorianCalendar dataLocacao = new GregorianCalendar(anoL, mesL, diaL);
						GregorianCalendar dataDevolucao = new GregorianCalendar(anoD, mesD, diaD);

						Periodo periodo = new Periodo(dataLocacao, dataDevolucao);
						
						if(!codigoCopia.isEmpty() && !cpfCliente.isEmpty() && !senha.isEmpty()){
							f.inserirLocacao(Long.parseLong(cpfCliente), Integer.parseInt(codigoCopia), periodo, senha);
							JOptionPane.showMessageDialog(this, "Locao inserida com sucesso", "Informao", JOptionPane.INFORMATION_MESSAGE);
						} else {
							throw new CampoInvalidoException();
						}
						
					}else if(this.funcao.equalsIgnoreCase("Devoluo Produto")){
						String codigoCopia = textoCodigoCopia.getText();
						if(!codigoCopia.isEmpty()){
							double custo = f.devolverProduto(Integer.parseInt(codigoCopia), Calendar.getInstance());
							if(custo >= 0){
								JOptionPane.showMessageDialog(this, "Locao inserida com sucesso;\nO custo total da locao foi: "+custo, "Informao", JOptionPane.INFORMATION_MESSAGE);
							}else{
								JOptionPane.showMessageDialog(this, "Locao inserida com sucesso;\nO sistema no pde calcular o custo porque no est online", "Informao", JOptionPane.INFORMATION_MESSAGE);
							}
						}else{
							throw new CampoInvalidoException();
						}
					}
					
					// TODO informar sucesso
					this.limparCampos();
					this.janelaPrincipal.trocarPainel(this.janelaPrincipal.getPainelInicial());
				}
			}catch(NumberFormatException nfe){
				JOptionPane.showMessageDialog(this, "Preecha o campo com nmeros", "Ateno", JOptionPane.ERROR_MESSAGE);
			} catch (PeriodoInvalidoException e) {
				JOptionPane.showMessageDialog(this, "O perodo selecionado est invlido", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (RemoteException e) {
				JOptionPane.showMessageDialog(this, "A filial est fora do ar", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (UsuarioNaoLogadoException e) {
				JOptionPane.showMessageDialog(this, "O usurio no est logado", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (UsuarioInexistenteException e) {
				JOptionPane.showMessageDialog(this, "O usurio no existe", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (UsuarioSemPermissaoException e) {
				JOptionPane.showMessageDialog(this, "O usurio np tem permisso para realizar esta operao", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (LocacaoInvalidaException e) {
				JOptionPane.showMessageDialog(this, "Locao invlida", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (LocacaoExistenteException e) {
				JOptionPane.showMessageDialog(this, "Locao j existente", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (CopiaLocadaNoPeriodoException e) {
				JOptionPane.showMessageDialog(this, "O plano selecionado no existe", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (CopiaReservadaNoPeriodoException e) {
				JOptionPane.showMessageDialog(this, "A cpia j est reservada neste perodo", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (OperacaoExistenteException e) {
				JOptionPane.showMessageDialog(this, "Operao j existente", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (IOException e) {
				JOptionPane.showMessageDialog(this, "Houve um problema de leitura de arquivo", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (OperacaoInvalidaException e) {
				JOptionPane.showMessageDialog(this, "Operao invlida", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (SenhaInvalidaException e) {
				JOptionPane.showMessageDialog(this, "Senha invlida", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (CopiaInexistenteException e) {
				JOptionPane.showMessageDialog(this, "Cpia inexistente", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (CampoInvalidoException e) {
				JOptionPane.showMessageDialog(this, "Preecha todos os campos", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (LocacaoInexistenteException e) {
				JOptionPane.showMessageDialog(this, "Locao inexistente", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (ProdutoInexistenteException e) {
				JOptionPane.showMessageDialog(this, "Produto inexistente", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (FilialInvalidaException e) {
				JOptionPane.showMessageDialog(this, "Filial invlida", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (FilialInexistenteException e) {
				JOptionPane.showMessageDialog(this, "Filial inexistente", "Erro!", JOptionPane.ERROR_MESSAGE);
			}
		}
	}
}

