package gui;

import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;

import negocio.exception.CampoInvalidoException;
import negocio.exception.EntregaInexistenteException;
import negocio.exception.ErroValidacaoException;
import negocio.exception.FilialNaoLogadaException;
import negocio.exception.OperacaoInexistenteException;
import negocio.exception.ReservaInexistenteException;

import dados.Entrega;
import dados.Operacao;
import dados.locacao.Reserva;

import util.IteratorBidirecional;

import fachada.Fachada;

public class PainelOperacoesEntregas extends PainelFigura {

	private static final long serialVersionUID = 1L;

	private JLabel labelOperacoes;
	private JLabel labelReserva;
	private JLabel labelEntrega;

	private JList listaOperacoes;
	private JList listaEntrega;
	private JList listaReserva;

	private JScrollPane barraOperacoes;
	private JScrollPane barraEntrega;
	private JScrollPane barraReserva;

	private JButton botaoOperacoes;
	private JButton botaoReserva;
	private JButton botaoEntrega;

	private String funcao;

	private IteratorBidirecional<Operacao> iteratorOperacoes;

	public PainelOperacoesEntregas(JanelaFilial janela){
		super("images/telaSecundaria.png", janela);

		this.funcao = "";

		// ** Operacoes **

		labelOperacoes = criarRotulo("Operaes Pendentes", 280, 60, 350, 50);
		botaoOperacoes = criarBotao("Remover",  325, 500, 120, 40);
		this.listaOperacoes = this.criarLista(30, 130, 700, 300);
		this.barraOperacoes = this.criarBarraRolagem(this.listaOperacoes);
		this.barraOperacoes.setVisible(false);
		labelOperacoes.setVisible(false);
		labelOperacoes.setFont(new Font("Arial",Font.BOLD ,20));

		// ** Entregas ***

		labelEntrega = criarRotulo("Entregas Pendentes", 130, 60, 200, 40);
		botaoEntrega = criarBotao("Remover", 180, 450, 120, 40);
		this.listaEntrega = this.criarLista(30, 150, 400, 250);
		this.barraEntrega = this.criarBarraRolagem(listaEntrega);
		this.barraEntrega.setVisible(false);
		this.labelEntrega.setVisible(false);
		labelEntrega.setFont(new Font("Arial",Font.BOLD ,20));

		// ** Reservas **

		labelReserva = criarRotulo("Notificao de Reservas", 480, 60, 400, 40);
		botaoReserva = criarBotao("Remover", 520, 450, 120, 40);
		this.listaReserva = this.criarLista(450, 150, 300, 250);
		this.barraReserva = this.criarBarraRolagem(listaReserva);
		this.barraReserva.setVisible(false);
		this.labelReserva.setVisible(false);
		labelReserva.setFont(new Font("Arial",Font.BOLD ,20));

	}

	public void carregarOperacoes(){
		DefaultListModel d = (DefaultListModel) this.listaOperacoes.getModel();
		d.clear();

		this.botaoOperacoes.setEnabled(true);

		try {
			iteratorOperacoes = Fachada.getInstancia(this.janelaPrincipal).getOperacoes();
			if(iteratorOperacoes != null){
				while(iteratorOperacoes.hasNext()){
					d.addElement(iteratorOperacoes.next());
				}
			}
		} catch (OperacaoInexistenteException e) {
			this.botaoOperacoes.setEnabled(false);
		}
	}


	public void carregarEntregas(){
		DefaultListModel d = (DefaultListModel) this.listaEntrega.getModel();
		d.clear();

		this.botaoEntrega.setEnabled(true);

		try {
			IteratorBidirecional<Entrega> iteratorEntregas = Fachada.getInstancia(this.janelaPrincipal).getEntregas();
			if(iteratorEntregas != null){
				while(iteratorEntregas.hasNext()){
					d.addElement(iteratorEntregas.next());
				}
			}
		} catch (EntregaInexistenteException e) {
			this.botaoEntrega.setEnabled(false);
		}
	}

	public void carregarReservas(){
		DefaultListModel d = (DefaultListModel) this.listaReserva.getModel();
		d.clear();

		this.botaoReserva.setEnabled(true);

		try {
			IteratorBidirecional<Reserva> iteratorReservas = Fachada.getInstancia(this.janelaPrincipal).getReservasNotificadas();
			if(iteratorReservas != null){
				while(iteratorReservas.hasNext()){
					d.addElement(iteratorReservas.next());
				}
			}
		} catch (ReservaInexistenteException e) {
			this.botaoReserva.setEnabled(false);
		}
	}


	public void operacoesAtivado(boolean a){
		barraOperacoes.setVisible(a);
		labelOperacoes.setVisible(a);
		botaoOperacoes.setVisible(a);
	}

	public void entregasReservasAtivado(boolean a){
		barraEntrega.setVisible(a);
		barraReserva.setVisible(a);
		labelEntrega.setVisible(a);
		labelReserva.setVisible(a);
		botaoReserva.setVisible(a);
		botaoEntrega.setVisible(a);
	}
	public void setFuncao(String funcao){
		boolean operacoesAtivado = false;

		if(funcao != null){
			if(funcao.equalsIgnoreCase("Operaes Pendentes")){
				operacoesAtivado = true;
			}else if(funcao.equalsIgnoreCase("Entregas Pendentes")){
				operacoesAtivado = false;
			}
		}
		this.operacoesAtivado(operacoesAtivado);
		this.entregasReservasAtivado(!operacoesAtivado);
		this.funcao = funcao;
	}

	public boolean validarOperacoes() {
		Fachada f = Fachada.getInstancia(this.janelaPrincipal);
		Operacao op = null;
		boolean ret = true;

		DefaultListModel d = (DefaultListModel) this.listaOperacoes.getModel();
		PrintWriter p = null;

		try {
			this.iteratorOperacoes = f.getOperacoes();

			try {
				p = new PrintWriter("erro validao.txt");

				while(iteratorOperacoes.hasNext()){
					try {
						op = iteratorOperacoes.next();
						f.validarOperacao(op);
						f.removerOperacao(op);
						d.removeElement(op);
						iteratorOperacoes.remove();
					} catch (ErroValidacaoException e) {
						p.write("Operao: " + op.toString() + "\n");
						p.write("Erro: " + e.getException().getMessage());
						ret = false;
					} catch (OperacaoInexistenteException e) {
						ret = false;
					} catch (IOException e) {
						ret = false;
					} catch (FilialNaoLogadaException e) {
						ret = false;
					}
				}
			} catch (FileNotFoundException e1) {
			}
		} catch (OperacaoInexistenteException e2) {
		}

		if(p != null){
			p.close();
		}

		return ret;
	}

	@Override
	public void actionPerformed(ActionEvent evento) {

		Component c = (Component) evento.getSource();
		DefaultListModel d;

		if(funcao != null){
			try{
				Fachada f = Fachada.getInstancia(this.janelaPrincipal);
				
				if(c == botaoOperacoes){
					Operacao op = (Operacao) this.listaOperacoes.getSelectedValue();

					if(op != null){
						d = (DefaultListModel) this.listaOperacoes.getModel();
						f.removerOperacao(op);
						d.removeElement(op);

						if(d.isEmpty()){
							this.botaoOperacoes.setEnabled(false);
						}
					} else {
						throw new CampoInvalidoException();
					}
				} else if(c == this.botaoReserva){
					Reserva res = (Reserva) this.listaReserva.getSelectedValue();
					
					if(res != null){
						d = (DefaultListModel) this.listaReserva.getModel();
						f.removerReservaNotificada(res.getCopia().getCodigoCopia());
						d.removeElement(res);
						
						if(d.isEmpty()){
							this.botaoReserva.setEnabled(false);
						}
					} else {
						throw new CampoInvalidoException();
					}

				} else if(c == this.botaoEntrega){
					Entrega ent = (Entrega) this.listaEntrega.getSelectedValue();

					if(ent != null){
						d = (DefaultListModel) this.listaEntrega.getModel();
						f.removerEntrega(ent.getLocacao().getCopia().getCodigoCopia());
						d.removeElement(ent);

						if(d.isEmpty()){
							this.botaoEntrega.setEnabled(false);
						}
					} else {
						throw new CampoInvalidoException();
					}

				}
			} catch (CampoInvalidoException e1) {
				JOptionPane.showMessageDialog(this, "Seleciona um elemento da lista antes de remov-lo.", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (OperacaoInexistenteException e2) {
				JOptionPane.showMessageDialog(this, "Operao inexistente", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (IOException e3) {
				JOptionPane.showMessageDialog(this, "Erro na leitura de arquivo", "Erro!", JOptionPane.ERROR_MESSAGE);
			}
		}
	}

}
