package gui;
import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.util.GregorianCalendar;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import negocio.exception.CampoInvalidoException;
import negocio.exception.CopiaInexistenteException;
import negocio.exception.CopiaLocadaNoPeriodoException;
import negocio.exception.CopiaReservadaNoPeriodoException;
import negocio.exception.FilialNaoLogadaException;
import negocio.exception.ProdutoInexistenteException;
import negocio.exception.ReservaExistenteException;
import negocio.exception.ReservaInexistenteException;
import negocio.exception.SenhaInvalidaException;
import negocio.exception.UsuarioInexistenteException;
import negocio.exception.UsuarioNaoLogadoException;
import negocio.exception.UsuarioSemPermissaoException;

import util.IteratorBidirecional;

import dados.exception.PeriodoInvalidoException;
import dados.exception.ReservaInvalidaException;
import dados.locacao.Periodo;
import dados.locacao.Reserva;
import fachada.Fachada;


public class PainelReserva extends PainelFigura {

	private static final long serialVersionUID = 1L;

	private String funcao;

	private JLabel labelCodigoCopia;
	private JLabel labelCPFCliente;
	private JLabel labelDataLocacao;
	private JLabel labelDataDevolucao;
	private JLabel labelTitulo;
	private JLabel labelNomeDia1;
	private JLabel labelNomeMes1;
	private JLabel labelNomeAno1;
	private JLabel labelNomeDia2;
	private JLabel labelNomeMes2;
	private JLabel labelNomeAno2;

	private JComboBox listaDia1;
	private JComboBox listaMes1;
	private JComboBox listaAno1;
	private JComboBox listaDia2;
	private JComboBox listaMes2;
	private JComboBox listaAno2;

	private JTextField textoCodigoCopia;
	private JTextField textoCPFCliente;
	private JButton botao;
	private JButton botaoRemover;
	private PainelSenha painelSenha;

	private JList lista;
	private JScrollPane barra;

	public PainelReserva(JanelaFilial janela){
		super("images/telaSecundaria.png", janela);

		this.funcao = "";

		labelTitulo = criarRotulo("", 0, 0, 200, 30);
		labelTitulo.setFont(new Font("Arial",Font.BOLD ,20));
		labelCodigoCopia = criarRotulo("Cdigo Cpia:", 40, 150, 150, 30);
		textoCodigoCopia = criarCampo(145, 150, 100, 25);
		labelCPFCliente = criarRotulo("CPF do Cliente:", 40, 200, 200, 30);
		textoCPFCliente = criarCampo(160, 200, 100, 25);
		botao = criarBotao("", 315, 500, 120, 40);
		botaoRemover = criarBotao("Remover", 450, 470, 120, 40);

		labelDataLocacao = criarRotulo("Data Incio Reserva:", 380, 150, 200, 30);
		labelDataDevolucao = criarRotulo("Data Fim Reserva:", 380, 250, 200, 30);

		labelNomeDia1 = criarRotulo("Dia", 535, 167, 50, 30);
		labelNomeMes1 = criarRotulo("Ms", 590, 167, 50, 30);
		labelNomeAno1 = criarRotulo("Ano", 655, 167, 50, 30);
		labelNomeDia2 = criarRotulo("Dia", 530, 267, 50, 30);
		labelNomeMes2 = criarRotulo("Ms", 590, 267, 50, 30);
		labelNomeAno2 = criarRotulo("Ano", 650, 267, 50, 30);

		listaDia1 = criarComboBox(525, 150, 50, 25);
		listaMes1 = criarComboBox(580, 150, 50, 25);
		listaAno1 = criarComboBox(635, 150, 70, 25);
		listaDia2 = criarComboBox(520, 250, 50, 25);
		listaMes2 = criarComboBox(575, 250, 50, 25);
		listaAno2 = criarComboBox(630, 250, 70, 25);

		criarData(listaDia1, listaMes1, listaAno1);
		criarData(listaDia2, listaMes2, listaAno2);

		labelNomeDia1.setFont(new Font("Arial",Font.PLAIN ,11));
		labelNomeMes1.setFont(new Font("Arial",Font.PLAIN ,11));
		labelNomeAno1.setFont(new Font("Arial",Font.PLAIN ,11));
		labelNomeDia2.setFont(new Font("Arial",Font.PLAIN ,11));
		labelNomeMes2.setFont(new Font("Arial",Font.PLAIN ,11));
		labelNomeAno2.setFont(new Font("Arial",Font.PLAIN ,11));

		this.lista = this.criarLista(300, 150, 420, 250);
		this.barra = this.criarBarraRolagem(this.lista);
		this.barra.setVisible(true);

		painelSenha = new PainelSenha(this.janelaPrincipal);
		this.add(painelSenha);
		painelSenha.setLocation(20,400);
		painelSenha.setVisible(true);


	}

	public void mostrarAtributosExtras(boolean mostrar){
		labelDataDevolucao.setVisible(mostrar);
		labelNomeDia2.setVisible(mostrar);
		labelNomeMes2.setVisible(mostrar);
		labelNomeAno2.setVisible(mostrar);
		listaDia2.setVisible(mostrar);
		listaMes2.setVisible(mostrar);
		listaAno2.setVisible(mostrar);
		labelCodigoCopia.setVisible(mostrar);
		textoCodigoCopia.setVisible(mostrar);
		labelDataLocacao.setVisible(mostrar);
		labelNomeDia1.setVisible(mostrar);
		labelNomeMes1.setVisible(mostrar);
		labelNomeAno1.setVisible(mostrar);
		listaDia1.setVisible(mostrar);
		listaMes1.setVisible(mostrar);
		listaAno1.setVisible(mostrar);
	}

	public void limpar(){
		textoCodigoCopia.setText("");
		textoCPFCliente.setText("");
		this.listaAno1.setSelectedIndex(0);
		this.listaAno2.setSelectedIndex(0);
		this.listaDia1.setSelectedIndex(0);
		this.listaDia2.setSelectedIndex(0);
		this.listaMes1.setSelectedIndex(0);
		this.listaMes2.setSelectedIndex(0);
		painelSenha.textoSenha.setText("");
	}

	public void setFuncao(String funcao){

		String titulo = "";
		String nomeBotao = "";
		boolean mostrarAtributosExtras = true;
		labelCPFCliente.setLocation(40, 200);
		textoCPFCliente.setLocation(160, 200);
		botao.setLocation(315, 500);
		botaoRemover.setVisible(false);
		barra.setVisible(false);

		if(funcao != null){
			if(funcao.equalsIgnoreCase("Fazer Reserva")){
				titulo = "Fazer Reserva";
				labelTitulo.setLocation(300,60);
				nomeBotao = "Reservar";
				painelSenha.setVisible(true);
			}else if(funcao.equalsIgnoreCase("Cancelar Reserva")){
				labelCPFCliente.setLocation(20, 150);
				textoCPFCliente.setLocation(140, 150);
				botao.setLocation(80, 230);
				titulo = "Cancelar Reserva";
				labelTitulo.setLocation(280,60);
				nomeBotao = "Buscar Reserva";
				mostrarAtributosExtras = false;
				painelSenha.setVisible(false);
				botaoRemover.setVisible(true);
				barra.setVisible(true);
				
				botaoRemover.setEnabled(false);
			}
		}
		labelTitulo.setText(titulo);
		botao.setText(nomeBotao);
		this.mostrarAtributosExtras(mostrarAtributosExtras);
		this.funcao = funcao;
	}

	@Override
	public void actionPerformed(ActionEvent a) {
		Component c = (Component) a.getSource();
		DefaultListModel d = (DefaultListModel) this.lista.getModel();

		if(this.funcao != null){
			try{
				Fachada f = Fachada.getInstancia(this.janelaPrincipal);
				if(c == botao){
					if(this.funcao.equalsIgnoreCase("Fazer Reserva")){
						String codigoCopia = textoCodigoCopia.getText();
						String cpfCliente = textoCPFCliente.getText();
						String senha = painelSenha.getSenha();

						Object diaLocacao = listaDia1.getSelectedItem();
						int diaL = Integer.parseInt(diaLocacao.toString());

						Object mesLocacao = listaMes1.getSelectedItem();
						int mesL = Integer.parseInt(mesLocacao.toString());

						Object anoLocacao = listaAno1.getSelectedItem();
						int anoL = Integer.parseInt(anoLocacao.toString());

						Object diaDevolucao = listaDia2.getSelectedItem();
						int diaD = Integer.parseInt(diaDevolucao.toString());

						Object mesDevolucao = listaMes2.getSelectedItem();
						int mesD = Integer.parseInt(mesDevolucao.toString());

						Object anoDevolucao = listaAno2.getSelectedItem();
						int anoD = Integer.parseInt(anoDevolucao.toString());

						GregorianCalendar dataLocacao = new GregorianCalendar(anoL, mesL, diaL);
						GregorianCalendar dataDevolucao = new GregorianCalendar(anoD, mesD, diaD);

						Periodo periodo = new Periodo(dataLocacao, dataDevolucao);
						if(!cpfCliente.isEmpty() && !senha.isEmpty() && !codigoCopia.isEmpty()){
							f.inserirReserva(Long.parseLong(cpfCliente), Integer.parseInt(codigoCopia), periodo, senha);
							JOptionPane.showMessageDialog(this, "Reserva inserida com sucesso", "Informao", JOptionPane.INFORMATION_MESSAGE);
						} else {
							throw new CampoInvalidoException();
						}
					}else if(this.funcao.equalsIgnoreCase("Cancelar Reserva")){
						String cpfCliente = textoCPFCliente.getText();
						String senha = painelSenha.getSenha();
						if(!cpfCliente.isEmpty() && !senha.isEmpty()){
							IteratorBidirecional<Reserva> iteratorReserva = f.buscarReservas(Integer.parseInt(cpfCliente));
							if(iteratorReserva != null){
								while(iteratorReserva.hasNext()){
									d.addElement(iteratorReserva.next());
								}
							}
						} else {
							throw new CampoInvalidoException();
						}
						
						botaoRemover.setEnabled(true);
					}
					this.limpar();
				}else if(c == botaoRemover){
					if(this.funcao.equalsIgnoreCase("Cancelar Reserva")){
						Reserva reserva = (Reserva) this.lista.getSelectedValue();
						if(reserva != null){
							f.removerReserva(reserva.getCliente().getCpf(), reserva.getCopia().getCodigoCopia());
							d.removeElement(reserva);
							JOptionPane.showMessageDialog(this, "Reserva removida com sucesso", "Informao", JOptionPane.INFORMATION_MESSAGE);
						} else {
							throw new CampoInvalidoException();
						}
					}
					this.limpar();
				}
			}

			catch(NumberFormatException nfe){
				JOptionPane.showMessageDialog(this, "Preecha o campo com nmeros", "Ateno", JOptionPane.ERROR_MESSAGE);
			} catch (PeriodoInvalidoException e) {
				JOptionPane.showMessageDialog(this, "Perodo invlido", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (UsuarioNaoLogadoException e) {
				JOptionPane.showMessageDialog(this, "O usurio no est logado", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (ReservaExistenteException e) {
				JOptionPane.showMessageDialog(this, "A reserva j existe", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (UsuarioSemPermissaoException e) {
				JOptionPane.showMessageDialog(this, "O usurio no tem permisso para realizar esta operao", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (CopiaLocadaNoPeriodoException e) {
				JOptionPane.showMessageDialog(this, "Nesse perodo, a cpia j est locada", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (CopiaReservadaNoPeriodoException e) {
				JOptionPane.showMessageDialog(this, "A cpia j est reservada neste perodo", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (FilialNaoLogadaException e) {
				JOptionPane.showMessageDialog(this, "Filial no logada", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (ReservaInvalidaException e) {
				JOptionPane.showMessageDialog(this, "Reserva invlida", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (UsuarioInexistenteException e) {
				JOptionPane.showMessageDialog(this, "Usurio inexistente", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (CopiaInexistenteException e) {
				JOptionPane.showMessageDialog(this, "Cpia inexistente", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (SenhaInvalidaException e) {
				JOptionPane.showMessageDialog(this, "Senha invlida", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (CampoInvalidoException e) {
				JOptionPane.showMessageDialog(this, "Preencha todos os campos", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (ReservaInexistenteException e) {
				JOptionPane.showMessageDialog(this, "Reserva inexistente", "Erro!", JOptionPane.ERROR_MESSAGE);
			} catch (ProdutoInexistenteException e) {
				JOptionPane.showMessageDialog(this, "Produto inexistente", "Erro!", JOptionPane.ERROR_MESSAGE);
			}
		}
	}
}
