package negocio;

import java.io.IOException;

import util.IteratorBidirecional;
import negocio.exception.OperacaoExistenteException;
import negocio.exception.OperacaoInexistenteException;
import dados.Operacao;
import armazenamento.RepositorioOperacao;

public class GerenciadorOperacoes {

	private static GerenciadorOperacoes instancia;
	private RepositorioOperacao repositorio;

	static{
		GerenciadorOperacoes.instancia = new GerenciadorOperacoes();
	}

	private GerenciadorOperacoes() {}

	public static GerenciadorOperacoes getInstancia(){
		instancia.repositorio = RepositorioOperacao.getInstancia();

		return GerenciadorOperacoes.instancia;
	}

	public void inserirOperacao(Operacao op) throws OperacaoExistenteException, IOException{
		if(op != null){
			if(!this.repositorio.existeOperacao(op)){
				this.repositorio.inserirOperacao(op);
			} else {
				throw new OperacaoExistenteException();
			}
		} else {
			throw new IllegalArgumentException();
		}
	}

	public boolean existeOperacao(Operacao op){
		if(op != null){
			return this.repositorio.existeOperacao(op);
		} else{
			throw new IllegalArgumentException();
		}
	}
	
	public boolean existeOperacao(){
		boolean retorno = true;
		
		try {
			this.getOperacoes();
		} catch (OperacaoInexistenteException e) {
			retorno = false;
		}
		
		return retorno;
	}

	public void removerOperacao(Operacao op) throws OperacaoInexistenteException, IOException{
		if(op != null){
			if(this.repositorio.existeOperacao(op)){
				this.repositorio.removerOperacao(op);
			} else {
				throw new OperacaoInexistenteException();
			}
		} else {
			throw new IllegalArgumentException();
		}
	}

	public IteratorBidirecional<Operacao> getOperacoes() throws OperacaoInexistenteException{
		IteratorBidirecional<Operacao> it = this.repositorio.getOperacoes();
		if(it.hasNext()){
			return it;
		} else {
			throw new OperacaoInexistenteException();
		}
	}

}
